//
//  ParameterSet.h
//  oneBuddy
//

#ifndef oneBuddy_ParameterSet_h
#define oneBuddy_ParameterSet_h
#include "Utilities.h"
#include "Term.h"
#include <vector>
#include <map>
#include <queue>
using namespace std;

#ifndef SMALL_VALUE
#define SMALL_VALUE  double(10e-14)
#endif

typedef class ParameterSet {
public:
    
    // HAMILTONIAN SYSTEM //////
    
    int verbose=3;
    double ETA=0.1;
    double OMEGA=0;
    
    vector<Term> termList;
    vector<Term> normalTermList;
    int nbOfTerms=0;
    int nbOfElements=0;
    
    // representation
    int nbOfBands=1;
    int Lx=1;
    int Ly=1;
    bool nambuTrigger=false;
    bool nambuTerm=false;
    int dim=1;
    int nambuDim=1;
    int normalDim=1;
    
    void readParameters(ifstream&);
    void readTerms(ifstream&);
    void readBilayer(ifstream&);
    void readElements(ifstream&,bool,bool);
    void print() const;
    
    
    // acting on termList
    bool isInTermList(string&) const;
    double getAmplitudeOf(const string&) const;
    void setAmplitudeOf(const string&, const double&);
    const Term& getTerm(const string&) const;
    
    void reduce();
    void setSlavesLists();
    
    void selfUpdate();
    void read(string);
    void readTask(ifstream&);
    
    
    // /////////////////////////
    
    
    
    
    
    // INTEGRATOR //////////////
    
    // cuba parameters
    double cuba_EPSREL=0;
    double cuba_EPSABS=0;
    int cuba_MAXEVAL=0;
    int cuba_MINEVAL=0;
    int cuba_verbose=0;
    
    // grid parameters
    double grid_EPSREL=0.001;
    double grid_EPSABS=0.00001;
    int grid_nMin=4;
    int grid_nMax=12;
    
    // /////////////////////////
    
    
    
    
    
    // LOOPER //////////////////
    
    // loop variables
    double loop_start=-1;
    double loop_end=1;
    double loop_incr=0.1;
    vector<double> loopValues;
    void readLOOP(istringstream&);
    
    // /////////////////////////
    
    
    
    
    
    // TASK// //////////////////
    
    // mdc parameters    
    bool mdc_trigger=false;
    bool mdc_gorgov=false;
    bool mdc_quarter=false;
    bool mdc_tiles=false;
    bool mdc_flipped=false;
    int mdc_numberOfKx=200;
    int mdc_numberOfKy=200;
    bool RBZmdc_trigger=false;
    // interact parameters
    bool mdc_interactive_trigger=false;
    int mdc=0;            // 0-MDC, 1-gorkovRE, 2-gorkovIM
    bool mdcFlip=false;   // 0-normal, 1-flip
    int other=0;          // 0-MDCs, 1-other
    vector<double> mdc_data;
    vector<double> gor_data_re;
    vector<double> gor_data_im;
    int mdc_totalNumberOfK = mdc_numberOfKx*mdc_numberOfKy;
    int lastUpdate=0;
    //
    void reset_mdc_data();
    void delete_mdc_data();
    
    // dos parameters
    bool dos_trigger=false;
    bool eigenDOS_trigger=false;
    
    // spectrum parameters
    bool spectrum_trigger=false;
    double spectrum_start_freq=-0.4;
    double spectrum_end_freq=0.2;
    double spectrum_freq_incr=0.001;
    int spectrum_wpoints=600;
    queue<double> spectrum_kx_queue;
    queue<double> spectrum_ky_queue;
    queue<int> spectrum_kpoints_queue;
    
    // density parameters
    bool density_trigger=false;
    bool density_trigger_loop=false;
    bool conductivity=false;
    bool conductivityLoop=false;
    bool conductivityLoop_parallel=false;
    double smallScale=0.2;
    double largeScale=10.0;
    double w_domain=1.0;
    double pole=3.0;
    double beta=50.0;
    
    // average parameters
    bool average_trigger=false;
    
    // localDOS
    bool localDOS_trigger=false;
    bool localDOSLoop_trigger=false;
    
    
    
    // hopping_DOS
    bool hdos_trigger=false;
    bool zmap = false;
    int hdos_size=0;
    int hdos_Xcells=1;
    int hdos_Ycells=1;
    vector<PlaneVector> neighborList;
    int bandSelected=0;
    int toBandSelected=0;
    // interactive
    bool hdos_interactive_trigger=false;
    vector<double> hdos_data;
    vector<double> hdos_data_neg;
    double hdos_max = 0;
    double hdos_min = 0;
    //
    void delete_hdos_data();
    
    
    
    // butterfly
    bool butterfly_trigger = false;
    string butt_parameter="B";
    PlaneVector butt_startQ = PlaneVector(0,1,0,1);
    PlaneVector butt_endQ = PlaneVector(1,1,0,1);
    int butt_Denom=200;
    double butt_pyc=0;
    string butt_filename="butterfly.out";
    double butt_pxcOff;
    
    // reading file
    void readMDC(istringstream&);
    void readCuba(istringstream&);
    void readGrid(istringstream&);
    void readSpectrum(istringstream&);
    void readButterfly(istringstream&);
    void readHoppingDOS(istringstream&);
    
    
} ParameterSet;


void ParameterSet::delete_hdos_data(){
    hdos_data.clear();
    hdos_data_neg.clear();
}


void ParameterSet::delete_mdc_data(){
    mdc_data.clear();
    gor_data_re.clear();
    gor_data_im.clear();
}

void ParameterSet::reset_mdc_data(){
    delete_mdc_data();
    mdc_totalNumberOfK = mdc_numberOfKx*mdc_numberOfKy;
    mdc_data.resize(mdc_numberOfKx*mdc_numberOfKy);
    gor_data_re.resize(mdc_numberOfKx*mdc_numberOfKy);
    gor_data_im.resize(mdc_numberOfKx*mdc_numberOfKy);
}


void ParameterSet::reduce(){
    for (auto& term : termList)
        for (auto& element : term.elementList) {
            element.reducePeriodFrac();
        }
}

void ParameterSet::setSlavesLists() {
    for (auto& termA : termList){
        for (string& slaveID : termA.slavesIdentifiers){
            for (auto& termB : termList) {
                if (termB.identifier == slaveID){
                    for (Element& element : termB.elementList){
                        termA.slavesAdresses.push_back( & element);
                    }
                }
            }
        }
        termA.setAmplitude(termA.factor);
    }
}

void ParameterSet::selfUpdate(){
    Lx = 1;
    Ly = 1;
    
    for (auto& term : termList) {
        nbOfTerms += 1;
        nbOfElements += (int)term.elementList.size();
        if (term.isNormal()) normalTermList.push_back(term);
        for (auto& element : term.elementList) {
            if (element.isSpin) nambuTrigger = true;
            if (element.isNambu) nambuTerm = nambuTrigger = true;
            if (element.band > nbOfBands) nbOfBands = element.band;
            if (element.toband > nbOfBands) nbOfBands = element.toband;
            
            Lx = lcm(Lx, element.waveVector.xDen);
            Ly = lcm(Ly, element.waveVector.yDen);
        }
    }
    
    normalDim = Lx*Ly;
    nambuDim = nambuTrigger? 2*normalDim : normalDim;
    dim = nambuDim*nbOfBands;
    
    for (auto& term : termList)
    for (auto& element : term.elementList) {
        element.forcePeriodXY(Lx,Ly);
    }
    
    setSlavesLists();
    
    hdos_size = 0;
    for (auto& hopping : neighborList) {
        if (hopping == PlaneVector(0,1,0,1)) hdos_size += normalDim;
        else hdos_size += 2*normalDim;
    }
    
    
}

void ParameterSet::read(string txtfilepath) {
    if (not exists(txtfilepath)) {cerr << endl << "ERROR: " << "couldn't find file '" << txtfilepath << "'" << endl << endl; exit(1);}
    
    if (verbose >= 1) cout << "reading parameters from " << txtfilepath << endl;
    ifstream file;
    file.open(txtfilepath);
    readTask(file);
    reset(file);
    readTerms(file);
    reset(file);
    for (auto& term : termList) {
        term.readElements(file);
        reset(file);
    }
}

void ParameterSet::readTask(ifstream& file) {
    reach(file,"task");
    nextLine(file);
    string line;
    while(getline(file,line)) {
        istringstream buffer(line);
        string word;
        buffer >> word;
        if (word=="mdc") {readMDC(buffer); mdc_trigger = true;}
        else if (word=="dos") {readLOOP(buffer); readCuba(buffer); dos_trigger = true;}
        else if (word=="density") {density_trigger=true;}
        else if (word=="average") {average_trigger=true;}
        else if (word=="conductivity") {readCuba(buffer); conductivity=true;}
        else if (word=="spectrum") {readSpectrum(buffer); spectrum_trigger=true;}
        else if (word=="localDOS") {readLOOP(buffer); readCuba(buffer); localDOSLoop_trigger=true;}
        else if (word=="density_loopMU") {readLOOP(buffer); readCuba(buffer); density_trigger_loop=true;}
        else if (word=="conductivity_loopMU") {readLOOP(buffer); readCuba(buffer); conductivityLoop=true;}
        else if (word=="conductivity_loopMU_parallel") {readLOOP(buffer); readCuba(buffer); conductivityLoop_parallel=true;}
        else if (word=="RBZmdc") {readMDC(buffer); RBZmdc_trigger = true;}
        else if (word=="butterfly") {readButterfly(buffer); butterfly_trigger = true;}
        else if (word=="eigenDOS") {readLOOP(buffer); readCuba(buffer); eigenDOS_trigger = true;}
        else if (word=="hoppingDOS") {readHoppingDOS(buffer); hdos_trigger = true;}
        else if (word=="cuba") {readCuba(buffer);}
        else if (word=="grid") {readGrid(buffer);}
        else if (word=="loop") {readLOOP(buffer);}
        else if (word=="") break;
        else if (word.front()=='#') {}
        else {cerr << endl << "ERROR: " << "unknown task '" << word << "'" << endl << endl; exit(1);}
    }
}

void ParameterSet::readLOOP(istringstream& buffer){
    string word;
    buffer >> word;
    
    istringstream rangeIn(word);
    char dummy[5];
    rangeIn >> dummy[0];
    rangeIn >> loop_start;
    rangeIn >> dummy[1];
    rangeIn >> loop_incr;
    rangeIn >> dummy[2];
    rangeIn >> loop_end;
    rangeIn >> dummy[3];
    dummy[4]='\0';
    
    if (not (strcmp(dummy,"(::)")==0)) {
        cerr << endl << "ERROR: ";
        cerr << "loop argument must be in the form (start:incr:end)";
        cerr << endl << endl;
        exit(1);
    }
    if (abs(loop_incr) < SMALL_VALUE) {
        cerr  << endl << "ERROR: " << "loop increment must be non zero" << endl;
        exit(1);
    }
    if ((loop_start < loop_end and loop_incr < 0) or (loop_end < loop_start and loop_incr > 0)) {
        cerr  << endl << "ERROR: " << "loop increment incompatible with start and end" << endl;
        exit(1);
    }
    
    loopValues.clear();
    for (double val=loop_start; (val-loop_start)/(loop_end-loop_start) < 1.00+SMALL_VALUE; val+=loop_incr) {
        loopValues.push_back(val);
    }
    
    
}

void ParameterSet::readHoppingDOS(istringstream& buffer){
    string word;
    neighborList.clear();
    neighborList.push_back(PlaneVector(0,1,0,1));
    bool isXset=false,isYset=false;
    while(buffer >> word) {
        cout << endl << endl << word.front() << endl << word[0] << endl << word << endl << endl;
        if (word.front()=='#') break;
        else if (word[0]=='(') {
            istringstream vecArg(word);
            PlaneVector newVec(vecArg);
            neighborList.push_back(newVec);
        }
        else if (is_uint(word)){
            int value=stoi(word);
            if (not isXset) {
                hdos_Xcells = value;
                hdos_Ycells = value;
                isXset = true;
            } else if (not isYset) {
                hdos_Ycells = value;
                isYset=true;
            } else{cerr << endl << "ERROR: " << "too many numbers in hoppingDOS arguments" << endl << endl; exit(1);}
        } else if (word == "band") {
            buffer >> bandSelected;
            buffer >> toBandSelected;
        } else {
            // if (word=="gorkov") mdc_gorgov=true;
            // else if (word=="quarter") mdc_quarter=true;
            //else if (word=="tiles") mdc_tiles=true;
            //else if (word=="flipped") mdc_flipped=true;
            //else
            {cerr << endl << "ERROR: " << "unknown keyword '" << word << "' in hoppingDOS arguments" << endl << endl; exit(1);}
        }
    }
}

void ParameterSet::readMDC(istringstream& buffer){
    string word;
    bool isXset=false,isYset=false;
    while(buffer >> word) {
        if (word.front()=='#') break;
        if (is_uint(word)){
            int value=stoi(word);
            if (not isXset) {
                mdc_numberOfKx = value;
                mdc_numberOfKy = value;
                isXset = true;
            } else if (not isYset) {
                mdc_numberOfKy = value;
                isYset=true;
            } else{cerr << endl << "ERROR: " << "too many numbers in MDC arguments" << endl << endl; exit(1);}
        } else {
            if (word=="gorkov") mdc_gorgov=true;
            else if (word=="quarter") mdc_quarter=true;
            else if (word=="tiles") mdc_tiles=true;
            else if (word=="flipped") mdc_flipped=true;
            else {cerr << endl << "ERROR: " << "unknown keyword '" << word << "' in MDC arguments" << endl << endl; exit(1);}
        }
    }
}

void ParameterSet::readCuba(istringstream& buffer){
    string word;
    while (buffer >> word) {
        if (word.front()=='#') break;
        if (word=="cuba_EPSREL"){
            if (not (buffer >> cuba_EPSREL)) {cerr << endl << "ERROR: " << "expecting double for cuba_EPSREL" << endl << endl; exit(1);}
        } else if (word=="cuba_EPSABS") {
            if (not (buffer >> cuba_EPSABS)) {cerr << endl << "ERROR: " << "expecting double for cuba_EPSABS" << endl << endl; exit(1);}
        } else if (word=="cuba_MINEVAL") {
            if (not (buffer >> cuba_MINEVAL)) {cerr << endl << "ERROR: " << "expecting integer for cuba_MINEVAL" << endl << endl; exit(1);}
        } else if (word=="cuba_MAXEVAL") {
            if (not (buffer >> cuba_MAXEVAL)) {cerr << endl << "ERROR: " << "expecting integer for cuba_MAXEVAL" << endl << endl; exit(1);}
        } else if (word=="cuba_verbose") {
            if (not (buffer >> cuba_verbose)) {cerr << endl << "ERROR: " << "expecting integer for cuba_verbose" << endl << endl; exit(1);}
            if ((cuba_verbose > 3) or (cuba_verbose < 0)) {cerr << endl << "ERROR: " << "expecting integer between 0 and 3 for cuba_verbose" << endl << endl; exit(1);}
        } else if (word=="EPSREL"){
            if (not (buffer >> cuba_EPSREL)) {cerr << endl << "ERROR: " << "expecting double for cuba_EPSREL" << endl << endl; exit(1);}
        } else if (word=="EPSABS") {
            if (not (buffer >> cuba_EPSABS)) {cerr << endl << "ERROR: " << "expecting double for cuba_EPSABS" << endl << endl; exit(1);}
        } else if (word=="MINEVAL") {
            if (not (buffer >> cuba_MINEVAL)) {cerr << endl << "ERROR: " << "expecting integer for cuba_MINEVAL" << endl << endl; exit(1);}
        } else if (word=="MAXEVAL") {
            if (not (buffer >> cuba_MAXEVAL)) {cerr << endl << "ERROR: " << "expecting integer for cuba_MAXEVAL" << endl << endl; exit(1);}
        } else if (word=="verbose") {
            if (not (buffer >> cuba_verbose)) {cerr << endl << "ERROR: " << "expecting integer for cuba_verbose" << endl << endl; exit(1);}
            if ((cuba_verbose > 3) or (cuba_verbose < 0)) {cerr << endl << "ERROR: " << "expecting integer between 0 and 3 for cuba_verbose" << endl << endl; exit(1);}
        } else {cerr << endl << "ERROR: " << "unknown keyword '" << word << "' in cuba arguments" << endl << endl; exit(1);}
    }
}


void ParameterSet::readGrid(istringstream& buffer){
    string word;
    while (buffer >> word) {
        if (word.front()=='#') break;
        if (word=="grid_EPSREL"){
            if (not (buffer >> grid_EPSREL)) {cerr << endl << "ERROR: " << "expecting double for grid_EPSREL" << endl << endl; exit(1);}
        } else if (word=="grid_EPSABS"){
            if (not (buffer >> grid_EPSABS)) {cerr << endl << "ERROR: " << "expecting double for grid_EPSABS" << endl << endl; exit(1);}
        } else if (word=="grid_nMin") {
            if (not (buffer >> grid_nMin)) {cerr << endl << "ERROR: " << "expecting integer for grid_nMin" << endl << endl; exit(1);}
        } else if (word=="grid_nMax") {
            if (not (buffer >> grid_nMax)) {cerr << endl << "ERROR: " << "expecting integer for grid_nMax" << endl << endl; exit(1);}
        } else {cerr << endl << "ERROR: " << "unknown keyword '" << word << "' in grid arguments" << endl << endl; exit(1);}
    }
}


void ParameterSet::readSpectrum(istringstream& buffer){
    string word;
    while (buffer >> word) {
        if (word.front()=='#') break;
        else if (word.front()=='w') {
            istringstream rangeIn(word);
            char dummy[6];
            rangeIn >> dummy[0];
            rangeIn >> dummy[1] >> spectrum_start_freq;
            rangeIn >> dummy[2] >> spectrum_freq_incr;
            rangeIn >> dummy[3] >> spectrum_end_freq;
            rangeIn >> dummy[4];
            dummy[5]='\0';
            if (not (strcmp(dummy,"w(::)")==0)) {cerr << endl << "ERROR: " << "spectrum w range should be written 'w(start:incr:end)'" << endl << endl; exit(1);}
        } else if (word.front()=='k') {
            istringstream rangeIn(word);
            char dummy[8];
            double kx; double ky; int kpoints;
            rangeIn >> dummy[0];
            rangeIn >> dummy[1] >> kx; spectrum_kx_queue.push(kx/2.0);
            rangeIn >> dummy[2] >> ky; spectrum_ky_queue.push(ky/2.0);
            rangeIn >> dummy[3];
            while (rangeIn >> kpoints){
                spectrum_kpoints_queue.push(kpoints);
                rangeIn >> dummy[4] >> kx; spectrum_kx_queue.push(kx/2.0);
                rangeIn >> dummy[5] >> ky; spectrum_ky_queue.push(ky/2.0);
                rangeIn >> dummy[6];
                dummy[7]='\0';
                if (not (strcmp(dummy,"k(,)(,)")==0)) {cerr << endl << "ERROR: " << "spectrum k range should be written 'k(kx1,ky1)n(kx2,ky2)'" << endl << endl; exit(1);}
            }
        }
    }
    spectrum_wpoints=(spectrum_end_freq-spectrum_start_freq)/spectrum_freq_incr + 1;
}

void ParameterSet::readButterfly(istringstream& buffer){
    buffer >> butt_parameter;
    butt_startQ = PlaneVector(buffer);
    butt_endQ = PlaneVector(buffer);
    buffer >> butt_Denom;
    buffer >> butt_pxcOff;
    buffer >> butt_pyc;
    buffer >> butt_filename;
}


void ParameterSet::readTerms(ifstream& file) {
    reach(file,"parameters");
    if (file.peek() == EOF) return;
    nextLine(file);
    string line;
    while(getline(file,line)) {
        
        istringstream buffer(line);
        string word; buffer >> word;
        if (word=="") break;
        else if (word.front()=='#') {}
        else if (word=="eta") buffer >> ETA;
        else if (word=="omega") buffer >> OMEGA;
        else if (not isInTermList(word)) {
            double value;
            string word2;
            vector<string> slaveTerms;
            while (buffer >> word2){
                if (word.front()=='#') break;
                else if (is_number(word2)) value = stod(word2);
                else slaveTerms.push_back(word2);
            }
            
            Term newTerm(word,value,slaveTerms);
            termList.push_back(newTerm);
            
        } else cout << endl << "WARNING: duplicate or bad input while reading Terms" << endl;
    }
}


bool ParameterSet::isInTermList(string& word) const{
    bool isIn = false;
    for (auto& term : termList) {
        if (term.identifier == word) {
            isIn=true;
            break;
        }
    }
    return isIn;
}

double ParameterSet::getAmplitudeOf(const string& word) const{
    for (auto& term : termList) {
        if (term.identifier == word) {
            return term.factor;
        }
    }
    return 0;
}

void ParameterSet::setAmplitudeOf(const string& word, const double& value){
    bool found = false;
    for (auto& term : termList){
        if (term.identifier == word) {
            term.setAmplitude(value);
            found = true;
            break;
        }
    }
    if (not found) throw out_of_range("Term '"+word+"' does not exist");
}

const Term& ParameterSet::getTerm(const string& word) const {
    for (auto& term : termList){
        if (term.identifier == word) {
            return term;
        }
    }
    // else
    throw out_of_range("Term '"+word+"' does not exist");
}







void ParameterSet::print() const{
    cout << endl;
    cout << "Inputed parameters are:" << endl;
    cout << "\tOMEGA \t" << OMEGA << endl;
    cout << "\tMU \t" << getAmplitudeOf("mu") << endl;
    cout << "\tETA \t" << ETA << endl;
    for (auto& term : termList){
        cout << "\t" << term.identifier << "\t" << term.factor << endl;
    }
    
    cout << "which yield terms:" << endl;
    for (auto& term : termList){
        for (auto& element : term.elementList){
            cout << "\t" << term.identifier;
            cout << "\t" << real(element.amplitude);
            cout << "\t(";
            cout << element.neighbourVector.xNum ;
            cout << ",";
            cout << element.neighbourVector.yNum ;
            cout << ")";
            if (element.waveVector.xDen != 1 or element.waveVector.yDen != 1) {
                cout << "\t(";
                cout << element.waveVector.xNum;
                if (element.waveVector.xDen != 1) cout << "/" << element.waveVector.xDen;
                cout << ",";
                cout << element.waveVector.yNum;
                if (element.waveVector.yDen != 1) cout << "/" << element.waveVector.yDen;
                cout << ")";
                cout << " " << element.phase;
            }
            cout << " \tband " << element.band << "-" << element.toband;
            if (element.isNambu) cout << " \tnambu";
            if (element.isSpin) cout << " \tspin DW";
            cout << endl;
        }
    }
    
    cout << "This lead to system size:" << endl;
    cout << "\tnbOfElements \t" << nbOfElements << endl;
    cout << "\tLx \t" << Lx << endl;
    cout << "\tLy \t" << Ly << endl;
    if (nambuTrigger) cout << "\twith spin down" << endl;
    cout << "\tnbOfBands \t" << nbOfBands << endl;
    cout << "\tnormalDim \t" << normalDim << endl;
    cout << "\tnambuDim \t" << nambuDim << endl;
    cout << "\ttotal dim \t" << dim << endl;
    
    if (mdc_trigger) {
        
        cout << "set MDC with:" << endl;
        cout << "\t\tkx points \t" << mdc_numberOfKx << endl;
        cout << "\t\tky points \t" << mdc_numberOfKy << endl;
        if (mdc_gorgov) cout << "\t\twith additional Gorkov function output" << endl;
        if (mdc_quarter) cout << "\t\tonly quarter of BZ" << endl;
        if (mdc_tiles) cout << "\t\twith tiling shown" << endl;
        if (mdc_flipped) cout << "\t\twith additional flipped output" << endl;
    }
    if (dos_trigger) {
        cout << "set DOS with:" << endl;
        cout << "\t\tstart \t" << loop_start << endl;
        cout << "\t\tincr \t" << loop_incr << endl;
        cout << "\t\tend \t" << loop_end << endl;
        if (cuba_EPSABS or cuba_EPSREL or cuba_MINEVAL or cuba_MAXEVAL){
            if (cuba_EPSREL!=0) cout << "\t\tcuba_EPSREL \t" << cuba_EPSREL << endl;
            else cout << "\t\tcuba_EPSREL \tdefault: 0.001" << endl;
            if (cuba_EPSABS!=0) cout << "\t\tcuba_EPSABS \t" << cuba_EPSABS << endl;        /// should read what is really in the code
            else cout << "\t\tcuba_EPSABS \tdefault: 0.0001" << endl;
            if (cuba_MINEVAL!=0) cout << "\t\tcuba_MINEVAL \t" << cuba_MINEVAL << endl;
            else cout << "\t\tcuba_MINEVAL \tdefault: 2.5/ETA^2" << endl;
            if (cuba_MAXEVAL!=0) cout << "\t\tcuba_MAXEVAL \t" << cuba_MAXEVAL << endl;
            else cout << "\t\tcuba_MAXEVAL \tdefault: 4 000000" << endl;
        }
    }
    if (spectrum_trigger) {
        queue<int> kpoints_queue = spectrum_kpoints_queue;
        queue<double> kx_queue = spectrum_kx_queue;
        queue<double> ky_queue = spectrum_ky_queue;
        
        double kx1 = kx_queue.front(); kx_queue.pop();
        double ky1 = ky_queue.front(); ky_queue.pop();
        while(!kpoints_queue.empty()){
            int kpoints = kpoints_queue.front(); kpoints_queue.pop();
            cout << "executing " << kpoints << " spectra, going from k=";
            cout << "(" << kx1 << "," << ky1 << ")";
            double kx2 = kx_queue.front(); kx_queue.pop();
            double ky2 = ky_queue.front(); ky_queue.pop();
            cout << " to k=(" << kx2 << "," << ky2 << ")" << endl;
            kx1 = kx2;
            ky1 = ky2;
        }
        cout << "\twith omega from " << spectrum_start_freq << " to " << spectrum_end_freq << " by steps of " << spectrum_freq_incr << endl;
    }
    if (butterfly_trigger) {
        cout << "set butterfly in '" << butt_filename << "' for term : " << butt_parameter << endl;
        cout << "\twith Denom :" << butt_Denom << " at pxc " << butt_pxcOff << " and pyc " << butt_pyc << endl;
        cout << "\t\tfrom: \t(" << butt_startQ.xNum << "/" << butt_startQ.xDen;
        cout << ","   << butt_startQ.yNum << "/" << butt_startQ.yDen << ")" << endl;
        cout << "\t\tto:   \t(" << butt_endQ.xNum << "/" << butt_endQ.xDen;
        cout << ","   << butt_endQ.yNum << "/" << butt_endQ.yDen << ")" << endl;
        
        
    }
    if (hdos_trigger) {
        cout << "set hoppingDOS with :'" << endl;
        cout << "\t\t" << hdos_Xcells << " unit cells in x " << endl;
        cout << "\t\t" << hdos_Ycells << " unit cells in y " << endl;
        cout << "\t\tlooping over " << endl;
        cout << "\t\tstart \t" << loop_start << endl;
        cout << "\t\tincr \t" << loop_incr << endl;
        cout << "\t\tend \t" << loop_end << endl;
    }
    if (mdc_interactive_trigger) { cout << "Interactive Mode" << endl;}
    
    
    
    cout << endl;
}

#endif
