//
//  PlaneVector.h
//  oneBuddy
//

#ifndef oneBuddy_PlaneVector_h
#define oneBuddy_PlaneVector_h
#include "Utilities.h"
using namespace std;



class PlaneVector {
public:
    bool isRational;
    int xNum;
    int xDen;
    int yNum;
    int yDen;
    double x;
    double y;
    
    
    PlaneVector();
    PlaneVector(const PlaneVector&);
    PlaneVector(istringstream&);
    void reduce();
    void update();
    PlaneVector(const double&,const double&);
    PlaneVector(const int&, const int&, const int&, const int&);
    bool operator==(const PlaneVector &) const;
    void forceDenomX(int);
    void forceDenomY(int);
    void forceDenomXY(int,int);     // all rational handling should be in a class Rational
    
    double operator*(const PlaneVector &);
    PlaneVector operator+=(const PlaneVector &);
    PlaneVector operator-=(const PlaneVector &);
    PlaneVector& operator*=(const double &);
    PlaneVector& operator*=(const int &);
    PlaneVector operator-() const;
    

private:
    void initRational(const int&, const int&, const int&, const int&);
};

PlaneVector::PlaneVector() {
    isRational = false;
    xNum = 0; yNum = 0;
    xDen = 0; yDen = 0;
    x = 0; y = 0;
}

PlaneVector::PlaneVector(PlaneVector const & v) {
    isRational = v.isRational;
    xNum = v.xNum; yNum = v.yNum;
    xDen = v.xDen; yDen = v.yDen;
    x = v.x; y = v.y;
}

PlaneVector::PlaneVector(const double &xval,const double &yval) {
    isRational = false;
    x = xval; y = yval;
    update();
}


void PlaneVector::initRational(const int &xn, const int &xd, const int &yn, const int &yd) {
    isRational = true;
    xNum = xn; yNum = yn;
    xDen = xd; yDen = yd;
    update();
}

PlaneVector::PlaneVector(const int &xn, const int &xd, const int &yn, const int &yd) {
    initRational(xn,xd,yn,yd);
}

PlaneVector::PlaneVector(istringstream &buffer){
    int xn,xd,yn,yd; char dummy[4];
    buffer >> dummy[0] >> xn >> dummy[1];
    if (dummy[1]=='/') buffer >> xd >> dummy[1];
    else xd=1;
    buffer >> yn >> dummy[2];
    if (dummy[2]=='/') buffer >> yd >> dummy[2];
    else yd=1;
    dummy[3]='\0';
    if (not (strcmp(dummy,"(,)")==0)) {cerr << endl << "ERROR: " << "failed reading vector" << endl << endl; exit(1);}
    initRational(xn,xd,yn,yd);
}

void PlaneVector::reduce() {
    if (isRational){
        int xgcd = abs(gcd(xNum,xDen));
        xNum = xNum/xgcd; xDen = xDen/xgcd;
        int ygcd = abs(gcd(yNum,yDen));
        yNum = yNum/ygcd; yDen = yDen/ygcd;
        x = xNum/(double)xDen; y = yNum/(double)yDen;
    }
}

//void PlaneVector::update() {
//    if (not isRational){
//        xNum=0;yNum=0;xDen=0;yDen=0;
//    } else
//        reduce();
//}


void PlaneVector::update() {
    if (isRational){
        int xgcd = abs(gcd(xNum,xDen));
        xNum = xNum/xgcd; xDen = xDen/xgcd;
        int ygcd = abs(gcd(yNum,yDen));
        yNum = yNum/ygcd; yDen = yDen/ygcd;
        x = xNum/(double)xDen; y = yNum/(double)yDen;
    }else{
        xNum=0;yNum=0;xDen=0;yDen=0;
    }
}














bool PlaneVector::operator==(const PlaneVector &v) const {
    if (isRational) {return (xNum == v.xNum and yNum == v.yNum and xDen == v.xDen and yDen == v.yDen);}
    else {return (dbEqual(x, v.x) and dbEqual(y, v.y));}
}

void PlaneVector::forceDenomXY(int Lx, int Ly){
    forceDenomX(Lx);
    forceDenomY(Ly);
}

void PlaneVector::forceDenomX(int Lx){
    if (!isRational){cerr << "cannot force denominator on irrational number" << endl; exit(1);}
    if ((Lx/xDen)!=floor(Lx/xDen)){cerr << "forced denom " << Lx << "was not compatible with " << xDen << endl; exit(1);}
    else {
        xNum = (Lx/xDen)*xNum;
        xDen = Lx;
    }
}

void PlaneVector::forceDenomY(int Ly){
    if (!isRational){cerr << "cannot force denominator on rational number" << endl; exit(1);}
    if ((Ly/yDen)!=floor(Ly/yDen)){cerr << "forced denom " << Ly << "was not compatible with " << yDen << endl; exit(1);}
    else {
        yNum = (Ly/yDen)*yNum;
        yDen = Ly;
    }
}

double PlaneVector::operator*(const PlaneVector & v) {
    return (x*v.x + y*v.y);
}

PlaneVector PlaneVector::operator+=(const PlaneVector &v) {
    if (isRational and v.isRational){
        xNum = xNum*v.xDen + v.xNum*xDen;
        xDen = xDen*v.xDen;
        yNum = yNum*v.yDen + v.yNum*yDen;
        yDen = yDen*v.yDen;
        update();
    }else{
        x = x+v.x;
        y = y+v.y;
    }
    return *this;
}

PlaneVector PlaneVector::operator-=(const PlaneVector & v) {
    if (isRational and v.isRational){
        xNum = xNum*v.xDen - v.xNum*xDen;
        xDen = xDen*v.xDen;
        yNum = yNum*v.yDen - v.yNum*yDen;
        yDen = yDen*v.yDen;
        update();
    }else{
        x = x-v.x;
        y = y-v.y;
    }
    return *this;
}

PlaneVector& PlaneVector::operator*=(const double &f) {
    isRational = false;
    x*=f; y*=f;
    update();
    return *this;
}

PlaneVector& PlaneVector::operator*=(const int &f) {
    if (isRational){
        xNum *= f;
        yNum *= f;
        update();
    }else{
        x*=f; y*=f;
    }
    return *this;
}

// implement division ?



PlaneVector operator+(PlaneVector vector, const PlaneVector &vectorB) {
    vector += vectorB;
    return vector;
}

PlaneVector operator*(PlaneVector vector, const double & factor) {
    vector *= factor;
    return vector;
}

PlaneVector operator*(const double & factor, const PlaneVector &vector) {
    return vector * factor;
}

PlaneVector operator*(PlaneVector vector, const int & factor) {
    vector *= factor;
    return vector;
}

PlaneVector operator*(const int & factor, const PlaneVector &vector) {
    return vector * factor;
}

PlaneVector PlaneVector::operator-() const {
    PlaneVector output(*this);
    output *= -1;
    return output;
}


#endif
