//
//  Element.h
//  oneBuddy
//

#ifndef oneBuddy_Term_h
#define oneBuddy_Term_h

#include <vector>
#include "Element.h"
#include "Utilities.h"
using namespace std;


class Term {
public:
    string identifier;
    double factor;
    vector<Element> elementList;
    vector<string> slavesIdentifiers;
    vector<Element*> slavesAdresses;
    
    Term();
    Term(const Term&);
    Term(const string&, const double&, vector<string>);
    void setAmplitude(double);
    void incrAmplitude(double);
    void addTerm(Element);
    void readElements(istream&);
    void readElements(istream&, const string&);
    bool isNormal() const;
};

Term::Term() {
    identifier="";
    factor=0;
}

Term::Term(const Term& t){
    identifier=t.identifier;
    factor=t.factor;
    elementList=t.elementList;
    slavesIdentifiers=t.slavesIdentifiers;
}

Term::Term(const string& name, const double& value, vector<string> names){
    identifier=name;
    factor=value;
    slavesIdentifiers=names;
}

void Term::setAmplitude(double newFactor){
    factor = newFactor;
    for (auto& element : elementList) {
        element.updateAmplitude(factor);
    }
    for (Element* slave : slavesAdresses) {
        slave->updatePhase(factor);
    }
}

void Term::incrAmplitude(double incr) {
    setAmplitude(factor+incr);
}

void Term::addTerm(Element element) {
    element.updateAmplitude(factor);
    elementList.push_back(element);
}

void Term::readElements(istream& file) {
    readElements(file,"normal"); reset(file);
    readElements(file,"nambu"); reset(file);
    readElements(file,"DW_spin"); reset(file);
    readElements(file,"DW_normal"); reset(file);
    readElements(file,"DW_nambu"); reset(file);
    readElements(file,"interband"); reset(file);
    if (identifier == "mu" and elementList.empty()){ // for retro compatibility
        double multiplier = -0.5;
        PlaneVector neighbour(0,1,0,1);
        elementList.push_back(Element(multiplier, neighbour, false,0,0));
        elementList.back().updateAmplitude(factor);
    }
}

void Term::readElements(istream& file, const string& keyword) {
    
    if (reach(file, keyword)) {
        bool isDW = (keyword.find("DW") != string::npos);
        bool isSpin = (keyword.find("spin") != string::npos);
        bool isNambu = (keyword.find("nambu") != string::npos);
        
        string line;
        getline(file,line);
        istringstream buffer(line);
        int band=0; buffer >> band;
        int toband=0; buffer >> toband;
        if (band != 0 and toband==0) toband=band;
        
        while(getline(file,line)) {
            istringstream buffer(line);
            string word;
            buffer >> word;
            
            if (word=="") break;
            if (word.front()=='#') continue;
            if (word==identifier) {
                double multiplier; buffer >> multiplier;
                PlaneVector neighbour(buffer);
                if (isDW) {
                    PlaneVector wavevector(buffer);
                    double phase; buffer >> phase;
                    elementList.push_back(Element(multiplier, neighbour, isNambu, wavevector, phase, isSpin, band, toband));
                }
                else elementList.push_back(Element(multiplier, neighbour, isNambu, band, toband));
                elementList.back().updateAmplitude(factor);
            }
        }
        
        readElements(file,keyword);
    }
    
}

bool Term::isNormal() const {
    bool outBool = true;
    for (auto& element : elementList) {
        if (not element.isNormal()) outBool = false ;
    }
    return outBool;
}


#endif
