//
//  average.h
//  oneBuddy
//

#ifndef oneBuddy_average_h
#define oneBuddy_average_h

#include "build.h"
#include "cuba.h"

void avg_iwk_Integrand(ParameterSet& p, const complex<double> z, const double pxc, const double pyc, double* output) {
    HmMatrix<complex<double>> greenUp(buildGreen(p, pxc, pyc, z));
    HmMatrix<complex<double>> greenDown(buildGreen(p, pxc, pyc, -z));
    
    for (int i =0; i<p.nbOfTerms; ++i) {
        HmMatrix<complex<double>> operatorMatrix(buildOperator(p, pxc, pyc, p.termList[i]));
        output[i] = 0;
        
        for (int band=0; band < p.nbOfBands; ++band) {
            
            int fromIndex = band*p.nambuDim;
            output[i] += real( (operatorMatrix * greenUp).partialTrace(p.normalDim, fromIndex)
                                       - operatorMatrix.partialTrace(p.normalDim, fromIndex) / (z-p.pole)
                             )/p.termList[i].factor/ (double)p.normalDim;
            
            if (p.nambuTrigger) {
                fromIndex = band*p.nambuDim + p.normalDim;
                output[i] += real( (operatorMatrix * greenDown).partialTrace(p.normalDim, fromIndex)
                                  + operatorMatrix.partialTrace(p.normalDim, fromIndex) / (z-p.pole)
                                  )/p.termList[i].factor / (double)p.normalDim;
            }
        }
        
        if (p.nambuTrigger) output[i] /= 2.0;
        
    }
}

static int avg_lowFreqCubaIntegrand(const int* ndim, const double x[],
                                const int* ncomp, double result[], void* userdata) {
    ParameterSet p = *((ParameterSet*)userdata);
    complex<double> z(0, x[0] * p.w_domain);
    avg_iwk_Integrand(p, z, x[1], x[2], result);
    return 0;
}

static int avg_midFreqCubaIntegrand(const int* ndim, const double x[],
                                const int* ncomp, double result[], void* userdata) {
    ParameterSet p = *((ParameterSet*)userdata);
    complex<double> z(0, x[0] * p.w_domain + p.smallScale);
    avg_iwk_Integrand(p, z, x[1], x[2], result);
    return 0;
}

static int avg_highFreqCubaIntegrand(const int* ndim, const double x[],
                                 const int* ncomp, double result[], void* userdata) {
    ParameterSet p = *((ParameterSet*)userdata);
    double iw = x[0] * p.w_domain;
    complex<double> z(0, 1.0/iw );
    avg_iwk_Integrand(p, z, x[1], x[2], result);
    for (int i=0; i<p.nbOfTerms; ++i) result[i] /= (iw*iw);
    return 0;
}


void avg_cubaIntegrateDavid(ParameterSet &p, double* integral, double* error, int verboseIntegrate=2){
    void* userdata=&p;
    int NDIM=3;                     // const    number of dimensions
    int ncomp=p.nbOfTerms;                    // const    number of components of the integrand (could be a vector) // here, it is the argument of var integral
    int nvec=1;                     // const    number of samples passed to integrand; usually 1 (here it can be larger than the one passed to the Integrand.. why? no one knows)
    double EPSREL=0.005;
    if (p.cuba_EPSREL!=0) EPSREL=p.cuba_EPSREL;
    double EPSABS=0.00001;
    if (p.cuba_EPSABS!=0) EPSABS=p.cuba_EPSABS;
    int verbose=p.cuba_verbose;                  // const    should be flags, but complicated, for verbose purpose, limits are from 0 to 3, higher values will throw other commands to Cuhre (refer to doc)
    int MINEVAL= 60000;//(int)(2.5/(p.ETA*p.ETA));      //(1000/((p.ETA*p.ETA)/(0.05*0.05)));
    if (p.cuba_MINEVAL!=0) MINEVAL=p.cuba_MINEVAL;
    int MAXEVAL=5000000;
    if (p.cuba_MAXEVAL!=0) MAXEVAL=p.cuba_MAXEVAL;
    int KEY=11;                     // const    7, 9, 11 or 13 (see doc) for 3D should be 11
    void* spin=NULL;                // const    deepshit to manage internal process (keep it NULL)
    int nregions;                   //      OUT actual number of subregion
    int neval;                      //      OUT number of evaluation
    int fail;                       //      OUT error flag (see doc)
    //double error[ncomp];            //      OUT error
    double prob[ncomp];             //      OUT another unreliable error (see doc)
    
    
    for (int i=0; i<p.nbOfTerms; ++i) {
        integral[i] = 0.0;
        error[i] = 0.0;
    }
    
    double *integralPortion = new double[p.nbOfTerms];
    double *errorPortion = new double[p.nbOfTerms];
    
    p.w_domain = p.smallScale;
    double accur = EPSABS*M_PI/p.w_domain;
    Cuhre(NDIM, ncomp, avg_lowFreqCubaIntegrand, userdata, nvec,
          EPSREL, accur, verbose, MINEVAL, MAXEVAL, KEY, NULL, spin,
          &nregions, &neval, &fail, integralPortion, errorPortion, prob);
    
    for (int i=0; i<p.nbOfTerms; ++i) {
        if(verboseIntegrate >= 2) printf(("Region 1 - <"+p.termList[i].identifier+">\t - Integral: %8.6f,   Error: %8.6f,   N_eval: %d \n").c_str(),integralPortion[i]*p.w_domain*M_1_PI, errorPortion[i], neval);
        integral[i] += integralPortion[i]*p.w_domain*M_1_PI;
        error[i] += abs(errorPortion[i]*p.w_domain*M_1_PI);
    }
    if(verboseIntegrate >= 2) printf("\n");
    
    p.w_domain = p.largeScale-p.smallScale;
    accur = EPSABS*M_PI/p.w_domain;
    Cuhre(NDIM, ncomp, avg_midFreqCubaIntegrand, userdata, nvec,
          EPSREL, accur, verbose, MINEVAL, MAXEVAL, KEY,  NULL, spin,
          &nregions, &neval, &fail, integralPortion, errorPortion, prob);
    
    for (int i=0; i<p.nbOfTerms; ++i) {
        if(verboseIntegrate >= 2) printf(("Region 2 - <"+p.termList[i].identifier+">\t - Integral: %8.6f,   Error: %8.6f,   N_eval: %d \n").c_str(),integralPortion[i]*p.w_domain*M_1_PI, errorPortion[i], neval);
        integral[i] += integralPortion[i]*p.w_domain*M_1_PI;
        error[i] += abs(errorPortion[i]*p.w_domain*M_1_PI);
    }
    if(verboseIntegrate >= 2) printf("\n");
    
    p.w_domain = 1.0/p.largeScale;
    accur = EPSABS*M_PI/p.w_domain;
    Cuhre(NDIM, ncomp, avg_highFreqCubaIntegrand, userdata, nvec,
          EPSREL, accur, verbose, MINEVAL, MAXEVAL, KEY,  NULL, spin,
          &nregions, &neval, &fail, integralPortion, errorPortion, prob);
    
    for (int i=0; i<p.nbOfTerms; ++i) {
        if(verboseIntegrate >= 2) printf(("Region 3 - <"+p.termList[i].identifier+">\t - Integral: %8.6f,   Error: %8.6f,   N_eval: %d \n").c_str(), integralPortion[i]*p.w_domain*M_1_PI, errorPortion[i], neval);
        integral[i] += integralPortion[i]*p.w_domain*M_1_PI;
        error[i] += abs(errorPortion[i]*p.w_domain*M_1_PI);
        integral[i] *= 2.0;
        error[i]    *= 2.0;
    }
    if(verboseIntegrate >= 2) printf("\n");
    
    delete[] integralPortion;
    delete[] errorPortion;
    
}

void average(ParameterSet &p){
    if(p.verbose >= 1) cout << endl << "calculating average, int over imaginary axis 'iw' (3 regions):" << endl;
    double *integral = new double[p.nbOfTerms]();
    double *error = new double[p.nbOfTerms]();
    
    avg_cubaIntegrateDavid(p,integral,error,p.verbose);
    
//    FILE *file = fopen("average.out","w");
//    fprintf(file," op     avg          err\n");
//    fprintf(file,"% 5.2f  % 8.6f  % 8.6f\n",p.termList[0].factor,integral[0],error[0]);
//    fclose(file);
    
    int i=0;
    for (auto& term : p.termList) {
        if(term.identifier == "mu") integral[i] *= -1.0; // because the operator is -mu
        if(p.verbose >= 1) printf(("<"+term.identifier+">\t = %8.6f \n").c_str(), integral[i]);
        i++;
    }
    
    delete[] integral;
    delete[] error;
}

#endif
