//
//  oneBuddy.h
//  OneBuddy
//

#ifndef OneBuddy_oneBuddy_h
#define OneBuddy_oneBuddy_h

#include "Utilities.h"
#include "HmMatrix.h"


complex<double> I(0.0,1.0);

HmMatrix<complex<double>> nambuPauli3(const ParameterSet& p) {
    HmMatrix<complex<double>> output=unit(p.dim);
    for (int i=p.normalDim; i < p.nambuDim; ++i) {
        for (int band=0; band < p.nbOfBands; ++band){
            int ii = i+band*p.nambuDim;
            output(ii,ii) *= -1;
        }
    }
    return output;
}


// COMMENTS of this section are for eventual Refactoring
void buildOperatorInPlace(HmMatrix<complex<double>>& outMatrix, const ParameterSet &p,
                          const double &px_continuous, const double &py_continuous,
                          const vector<Term>& termList, const int& dx=0, const int& dy=0 ){
    
    // determine 00 wave vector : ( p, pxc, pxy )
    double kx = 2*M_PI*( px_continuous/(double)p.Lx - 0.5 ); // are those -0.5 really necessary?
    double ky = 2*M_PI*( py_continuous/(double)p.Ly - 0.5 );
    PlaneVector k0( kx , ky );
    // ////////////////////////////
    
    outMatrix.resize(p.dim);
    outMatrix.setToZeros();
    
    for (int nx=0; nx<p.Lx; ++nx) {
        for (int ny=0; ny<p.Ly; ++ny) {
            
            
            // DOES NOT DEPEND ON pxc and pyc COULD BE OUT OF HERE
            // determine offset wave vector : ( p, nx, ny )
            double kxn = 2*M_PI*nx/(double)p.Lx;
            double kyn = 2*M_PI*ny/(double)p.Ly;
            PlaneVector kn( kxn , kyn );
            // depends only on the nature of p could be stored
            // ///////////////////////////////////////////////////
            
            
            
            // the terms in term list will define the operator
            for (auto& term : termList) {
                for (int band=0; band < p.nbOfBands; ++band){
                    for (auto& element : term.elementList) {
                        if (element.band !=0 and element.band != band+1) continue;
                        
                        
                        // DOES NOT DEPEND ON pxc and pyc COULD BE OUT OF HERE
                        // element.SetIndexMapForRepKKkForSystem( p )
                        // determines i,j,jn : (p, element) + for on (nx, ny, band)
                        int qx = element.waveVector.xNum * p.Lx / element.waveVector.xDen;        // integer waveVector numerator over Lx (could be encapsulated when updating ParameterSet object)
                        int qy = element.waveVector.yNum * p.Ly / element.waveVector.yDen;
                        int i,j,jn;
                        // case interband : WARNING : may need to reconsider nambu+interband or Q+interband terms
                        if (element.toband != element.band) {
                            i = p.Ly* nx + ny                                   +((element.band-1)*p.nambuDim);
                            j = p.Ly* posMod(nx+qx,p.Lx) + posMod(ny+qy,p.Ly)   +((element.toband-1)*p.nambuDim);
                            jn = p.Ly* posMod(nx-qx,p.Lx) + posMod(ny-qy,p.Ly)  +((element.toband-1)*p.nambuDim);
                        } else {
                            i = p.Ly* nx + ny                                   +(band*p.nambuDim);
                            j = p.Ly* posMod(nx+qx,p.Lx) + posMod(ny+qy,p.Ly)   +(band*p.nambuDim);
                            jn = p.Ly* posMod(nx-qx,p.Lx) + posMod(ny-qy,p.Ly)  +(band*p.nambuDim);
                        }
                        // depends only on the nature of the element and p. +1 for an element factory
                        // ////////////////////////////////////////////////////
                        
                        
                        
                        // //// loose shit tying everything togheter...
                        PlaneVector k(k0 + kn);
                        // //////////////
                        // because i, j, jn, must be synced with kn, through nx,ny
                        // may be this link is artificial, because of the periodicity of k-space...
                        // may be it has to do with the -0.5 of the def of k0...
                        
                        
                        // element.addItsValueForKTo( hamiltonian, k, p ) {
                        // for indices in element.indexMap
                        // hamiltonian(indices) = p.factorMap(indices)*element.value( k + p.kOffsetMap(indices) )
                        // }
                        // add element value in matrix : needs  (i, j, jn, k, element)
                        if (element.isNambu){
                            outMatrix(i,j+p.normalDim) += -conj(element.value(k, dx,dy ));    //nambu 1,0
                            outMatrix(j+p.normalDim,i) += -element.value(k, dx,dy );          //nambu 0,1
                        }
                        else {
                            outMatrix(i,j) += conj(element.value(k, dx,dy ));
                            outMatrix(j,i) += element.value(k, dx,dy );
                            
                            if (p.nambuTrigger) {
                                if (element.isSpin){
                                    outMatrix(i+p.normalDim, jn+p.normalDim) += element.value(-k, dx,dy );
                                    outMatrix(jn+p.normalDim, i+p.normalDim) += conj(element.value(-k, dx,dy ));
                                } else {
                                    outMatrix(i+p.normalDim, jn+p.normalDim) += -element.value(-k, dx,dy );
                                    outMatrix(jn+p.normalDim, i+p.normalDim) += -conj(element.value(-k, dx,dy ));
                                }
                            }
                        }
                        // depends mainly of the nature of the element. +1 for an element factory
                        
                    }
                }
            }
        }}
}

void checkOperatorInPlace(HmMatrix<bool>& outMatrix, const ParameterSet &p, const vector<Term>& termList ){
    
    outMatrix.resize(p.dim);
    outMatrix.setToZeros();
    
    for (int nx=0; nx<p.Lx; ++nx){
        for (int ny=0; ny<p.Ly; ++ny) {

            // the terms in term list will define the operator
            for (auto& term : termList) {
                for (int band=0; band < p.nbOfBands; ++band) {
                    for (auto& element : term.elementList) {
                        if (element.band !=0 and element.band != band+1) continue;
                        
                        // determines i,j,jn : (p, element) + for on (nx, ny, band)
                        int qx = element.waveVector.xNum * p.Lx / element.waveVector.xDen;        // integer waveVector numerator over Lx (could be encapsulated when updating ParameterSet object)
                        int qy = element.waveVector.yNum * p.Ly / element.waveVector.yDen;

                        // case interband : WARNING : may need to reconsider nambu+interband or Q+interband terms
                        int i,j,jn;
                        if (element.toband != element.band) {
                            i = p.Ly* nx + ny                                   +((element.band-1)*p.nambuDim);
                            j = p.Ly* posMod(nx+qx,p.Lx) + posMod(ny+qy,p.Ly)   +((element.toband-1)*p.nambuDim);
                            jn = p.Ly* posMod(nx-qx,p.Lx) + posMod(ny-qy,p.Ly)  +((element.toband-1)*p.nambuDim);
                        } else {
                            i = p.Ly* nx + ny                                   +(band*p.nambuDim); // because element.band==0 or element.band-1 == band
                            j = p.Ly* posMod(nx+qx,p.Lx) + posMod(ny+qy,p.Ly)   +(band*p.nambuDim);
                            jn = p.Ly* posMod(nx-qx,p.Lx) + posMod(ny-qy,p.Ly)  +(band*p.nambuDim);
                        }

                        // add element value in matrix : needs  (i, j, jn, k, element)
                        if (element.isNambu){
                            outMatrix(i,j+p.normalDim) = true;    //nambu 1,0
                            outMatrix(j+p.normalDim,i) = true;    //nambu 0,1
                        }
                        else {
                            outMatrix(i,j) = true;
                            outMatrix(j,i) = true;
                            
                            if (p.nambuTrigger) {
                                if (element.isSpin){
                                    outMatrix(i+p.normalDim, jn+p.normalDim) = true;
                                    outMatrix(jn+p.normalDim, i+p.normalDim) = true;
                                } else {
                                    outMatrix(i+p.normalDim, jn+p.normalDim) = true;
                                    outMatrix(jn+p.normalDim, i+p.normalDim) = true;
                                }
                            }
                        }
                    }
                }
            }
        }
    }
}

HmMatrix<bool> checkHamiltonian(const ParameterSet &p){
    HmMatrix<bool> matrix;
    checkOperatorInPlace(matrix, p, p.termList);
    return matrix;
}


HmMatrix<complex<double>> buildOperator(const ParameterSet &p, const double &px_continuous, const double &py_continuous, const vector<Term>& termList){
    HmMatrix<complex<double>> operatorMatrix;
    buildOperatorInPlace(operatorMatrix, p, px_continuous, py_continuous, termList);
    return operatorMatrix;
}

HmMatrix<complex<double>> buildOperator(const ParameterSet &p, const double &px_continuous, const double &py_continuous, const Term& term){
    vector<Term> oneTermList;
    oneTermList.push_back(term);
    return buildOperator(p, px_continuous, py_continuous, oneTermList);
}

void buildHamiltonianInPlace(HmMatrix<complex<double>>& hamiltonian, const ParameterSet &p,
                             const double &px_continuous, const double &py_continuous, 
                             const int& dx=0, const int& dy=0){
    buildOperatorInPlace(hamiltonian, p, px_continuous, py_continuous, p.termList, dx, dy);
}

HmMatrix<complex<double>> buildHamiltonian(const ParameterSet &p, const double &px_continuous, const double &py_continuous){
    HmMatrix<complex<double>> hamiltonian;
    buildHamiltonianInPlace(hamiltonian, p, px_continuous, py_continuous);
    return hamiltonian;
}

void buildGreenInPlace(HmMatrix<complex<double>>& green, const ParameterSet &p, const double &px_continuous, const double &py_continuous, const complex<double> z){
    buildOperatorInPlace(green,p,px_continuous,py_continuous, p.termList);
    green *= (complex<double>)-1.0;
    green += z*unit(p.dim);
    green.selfInverse();
}

HmMatrix<complex<double>> buildGreen(const ParameterSet &p, const double &px_continuous, const double &py_continuous, const complex<double> z){
    HmMatrix<complex<double>> green;
    buildGreenInPlace(green, p, px_continuous, py_continuous, z);
    return green;
}






void buildOperatorRepRRInPlace(HmMatrix<complex<double>>& outMatrix, const ParameterSet &p,
                               const double &px_continuous, const double &py_continuous,
                               const vector<Term>& termList, const int& dx=0, const int& dy=0 ){
    
    outMatrix.resize(p.dim);
    outMatrix.setToZeros();
    
    double kx = 2*M_PI*( px_continuous/(double)p.Lx - 0.5 ); // are those -0.5 really necessary?
    double ky = 2*M_PI*( py_continuous/(double)p.Ly - 0.5 );
    PlaneVector k( kx , ky );
    
    for (int Rx=0; Rx<p.Lx; ++Rx) {
    for (int Ry=0; Ry<p.Ly; ++Ry) {
        PlaneVector R( Rx,1, Ry,1 );
        
        
        for (auto& term : termList) {
        for (int band=0; band < p.nbOfBands; ++band){
        for (auto& element : term.elementList) {
            if (element.band !=0 and element.band != band+1) continue;
                
            int ax = element.neighbourVector.xNum;
            int ay = element.neighbourVector.yNum;
            
            int i,j;
            if (element.toband != element.band) {
                i = p.Ly* Rx + Ry                                   +((element.band-1)*p.nambuDim);
                j = p.Ly* posMod(Rx+ax,p.Lx) + posMod(Ry+ay,p.Ly)   +((element.toband-1)*p.nambuDim);
            } else {
                i = p.Ly* Rx + Ry                                   +(band*p.nambuDim); // because element.band==0 or element.band-1 == band
                j = p.Ly* posMod(Rx+ax,p.Lx) + posMod(Ry+ay,p.Ly)   +(band*p.nambuDim);
            }
            
            if (element.isNambu){
                outMatrix( i+p.normalDim, j ) += -element.valueRR(-k, R, p.Lx, p.Ly);
                outMatrix( j+p.normalDim, i ) += -element.valueRR( k, R, p.Lx, p.Ly);
                outMatrix( j, i+p.normalDim ) += -conj(element.valueRR(-k, R, p.Lx, p.Ly));
                outMatrix( i, j+p.normalDim ) += -conj(element.valueRR( k, R, p.Lx, p.Ly));
            }
            else {
                outMatrix(j,i) += element.valueRR(k, R, p.Lx, p.Ly);
                outMatrix(i,j) += conj( element.valueRR(k, R, p.Lx, p.Ly) );
                
                if (p.nambuTrigger) {
                    outMatrix(i+p.normalDim, j+p.normalDim) += -element.valueRR(-k, R, p.Lx, p.Ly);
                    outMatrix(j+p.normalDim, i+p.normalDim) += -conj(element.valueRR(-k, R, p.Lx, p.Ly));
                }
            }
            
        }}}
    }}
}


void buildHamiltonianRepRRInPlace(HmMatrix<complex<double>>& hamiltonian, const ParameterSet &p,
                             const double &px_continuous, const double &py_continuous){
    buildOperatorRepRRInPlace(hamiltonian, p, px_continuous, py_continuous, p.termList);
}

HmMatrix<complex<double>> buildHamiltonianRepRR(const ParameterSet &p, const double &px_continuous, const double &py_continuous){
    HmMatrix<complex<double>> hamiltonian;
    buildHamiltonianRepRRInPlace(hamiltonian, p, px_continuous, py_continuous);
    return hamiltonian;
}

void buildGreenRepRRInPlace(HmMatrix<complex<double>>& green, const ParameterSet &p, const double &px_continuous, const double &py_continuous, const complex<double> z){
    buildOperatorRepRRInPlace(green,p,px_continuous,py_continuous, p.termList);
    green *= (complex<double>)-1.0;
    green += z*unit(p.dim);
    green.selfInverse();
}

HmMatrix<complex<double>> buildGreenRepRR(const ParameterSet &p, const double &px_continuous, const double &py_continuous, const complex<double> z){
    HmMatrix<complex<double>> green;
    buildGreenRepRRInPlace(green, p, px_continuous, py_continuous, z);
    return green;
}


#endif
