//
//  conductivity.h
//  oneBuddy
//

#ifndef oneBuddy_conductivity_h
#define oneBuddy_conductivity_h

#include "build.h"
#include "cuba.h"
#include "density.h"
#include <exception>
//#include <omp.h>

// Conductivity

vector<set<int>> findSectors(HmMatrix<bool> matrixBool) {
    set<int> indexBank;
    vector<set<int>> sectors; 
    for(unsigned int i=0; i<matrixBool.dim(); i++) indexBank.insert(i);
    //for(set<int>::iterator iter=indexBank.begin(); iter!=indexBank.end();++iter) cout<<(*iter)<<" ";
    //cout<<"\n";
    int NN=0;
    while(indexBank.size()!=0)
    {   
        NN+=1;
        set<int> sectorList;
        set<int> sectorListNext;
        
        sectorList.clear();
        sectorListNext.clear();
        
        int element = *indexBank.begin();
        sectorListNext.insert(element);
        indexBank.erase(element);
        
        while(sectorList.size() != sectorListNext.size())
        {
            sectorList = sectorListNext;
            for(set<int>::iterator i1=sectorList.begin(); i1!=sectorList.end();++i1)
            {   
                int i = *i1;
                for(set<int>::iterator i2=indexBank.begin(); i2!=indexBank.end();++i2)
                {
                    int j = *i2;
                    if(matrixBool(i,j) || matrixBool(j,i)) if(sectorListNext.find(j) == sectorListNext.end()) sectorListNext.insert(j);
                }
                for(set<int>::iterator iter=sectorListNext.begin(); iter!=sectorListNext.end();++iter) if(indexBank.find(*iter) != indexBank.end()) indexBank.erase(*iter);
            }
        }
        sectors.push_back(sectorList);
    }
    return sectors;
}


//////////////////////
class HamiltonianByProducts{
public:
    HamiltonianByProducts(ParameterSet &p);

    vector<vector<double>> eigenValues;
    vector<HmMatrix<complex<double>>> eigenVectors;
    vector<HmMatrix<complex<double>>> eigenVectorsDag;
    vector<HmMatrix<complex<double>>> Hkyky;
    vector<HmMatrix<complex<double>>> Hkxkx;
    vector<HmMatrix<complex<double>>> Hkxky;
    vector<HmMatrix<complex<double>>> Hkx;
    vector<HmMatrix<complex<double>>> Hky;
    vector<HmMatrix<complex<double>>> hamiltonianSector;
    vector<vector<complex<double>>> lambdaX;
    vector<vector<complex<double>>> lambdaY;
    vector<vector<complex<double>>> lambdaXX;
    vector<vector<complex<double>>> lambdaYY;
    vector<vector<complex<double>>> lambdaXY;
    vector<vector<complex<double>>> partialLambdaXX;
    vector<vector<complex<double>>> partialLambdaXY;
    vector<vector<complex<double>>> partialLambdaYY;
    vector<set<int>> sectors;
    HmMatrix<complex<double>> temp;
    HmMatrix<complex<double>> hamiltonian;
};

HamiltonianByProducts::HamiltonianByProducts(ParameterSet &p){

    HmMatrix<bool> hamiltonianTerms = checkHamiltonian(p);    
    sectors= findSectors(hamiltonianTerms);
    
    int sz = (int)sectors.size();
    hamiltonianSector.resize(sz);
    lambdaX.resize(sz); lambdaY.resize(sz); 
    lambdaXX.resize(sz); lambdaXY.resize(sz); lambdaYY.resize(sz);
    partialLambdaXX.resize(sz); partialLambdaXY.resize(sz); partialLambdaYY.resize(sz);
    eigenValues.resize(sz); eigenValues.resize(sz);
    eigenVectors.resize(sz); eigenVectorsDag.resize(sz);
    Hkyky.resize(sz); Hkxkx.resize(sz); Hkxky.resize(sz);
    Hkx.resize(sz); Hky.resize(sz);
    
    for(unsigned int i=0; i<sectors.size(); i++)
    {   
        sz = (int)sectors[i].size();
        hamiltonianSector[i].resize(sz);
        lambdaX[i].resize(sz); lambdaY[i].resize(sz); 
        lambdaXX[i].resize(sz); lambdaXY[i].resize(sz); lambdaYY[i].resize(sz);
        partialLambdaXX[i].resize(sz); partialLambdaXY[i].resize(sz); partialLambdaYY[i].resize(sz);
        eigenValues[i].resize(sz); eigenValues[i].resize(sz);
        eigenVectors[i].resize(sz); eigenVectorsDag[i].resize(sz);
        Hkyky[i].resize(sz); Hkxkx[i].resize(sz); Hkxky[i].resize(sz);
        Hkx[i].resize(sz); Hky[i].resize(sz);
        
    }
}
/////////////////////////////


static void calculateConductivityInPlace(ParameterSet &p, HamiltonianByProducts &h,const double pxc,const double pyc){
    
    buildHamiltonianInPlace(h.hamiltonian,p,pxc,pyc,0,0);
    for(unsigned int i=0; i<h.sectors.size(); i++){
        h.hamiltonian.squareBloc(h.hamiltonianSector[i], h.sectors[i]);
        // If sector in Nambu space, take minus the conjugate of the hamiltonian.
        // Note that we take for granted that the hamiltonian does not have any components in the Gorkov-Green function
        // (since we imposed the condition: p.nambuTerm = false).
        if(*h.sectors[i].begin() >= p.normalDim) h.hamiltonianSector[i].selfConj() *= complex<double>(-1.0,0.0);  
        diagonalize(h.hamiltonianSector[i], h.eigenVectors[i], h.eigenValues[i]);
        h.eigenVectorsDag[i] = h.eigenVectors[i].adjoint();
    }
    buildHamiltonianInPlace(h.hamiltonian,p,pxc,pyc,1,0);
    for(unsigned int i=0; i<h.sectors.size(); i++){
        h.hamiltonian.squareBloc(h.Hkx[i], h.sectors[i]);
        if(*h.sectors[i].begin() >= p.normalDim) h.Hkx[i].selfConj() *= complex<double>(-1.0,0.0); //same as above
        h.Hkx[i] = h.eigenVectorsDag[i] * h.Hkx[i] * h.eigenVectors[i];
    }
    buildHamiltonianInPlace(h.hamiltonian,p,pxc,pyc,0,1);
    for(unsigned int i=0; i<h.sectors.size(); i++){
        h.hamiltonian.squareBloc(h.Hky[i], h.sectors[i]);
        if(*h.sectors[i].begin() >= p.normalDim) h.Hky[i].selfConj() *= complex<double>(-1.0,0.0); //same as above
        h.Hky[i] = h.eigenVectorsDag[i] * h.Hky[i] * h.eigenVectors[i];
    }
    buildHamiltonianInPlace(h.hamiltonian,p,pxc,pyc,2,0);
    for(unsigned int i=0; i<h.sectors.size(); i++){
        h.hamiltonian.squareBloc(h.Hkxkx[i], h.sectors[i]);
        if(*h.sectors[i].begin() >= p.normalDim) h.Hkxkx[i].selfConj() *= complex<double>(-1.0,0.0); //same as above
        h.Hkxkx[i] = h.eigenVectorsDag[i] * h.Hkxkx[i] * h.eigenVectors[i];
    }
    buildHamiltonianInPlace(h.hamiltonian,p,pxc,pyc,0,2);
    for(unsigned int i=0; i<h.sectors.size(); i++){
        h.hamiltonian.squareBloc(h.Hkyky[i], h.sectors[i]);
        if(*h.sectors[i].begin() >= p.normalDim) h.Hkyky[i].selfConj() *= complex<double>(-1.0,0.0); //same as above
        h.Hkyky[i] = h.eigenVectorsDag[i] * h.Hkyky[i] * h.eigenVectors[i];
    }
    buildHamiltonianInPlace(h.hamiltonian,p,pxc,pyc,1,1);
    for(unsigned int i=0; i<h.sectors.size(); i++){
        h.hamiltonian.squareBloc(h.Hkxky[i], h.sectors[i]);
        if(*h.sectors[i].begin() >= p.normalDim) h.Hkxky[i].selfConj() *= complex<double>(-1.0,0.0); //same as above
        h.Hkxky[i] = h.eigenVectorsDag[i] * h.Hkxky[i] * h.eigenVectors[i];
    }
    
    for(unsigned int i=0; i<h.sectors.size(); i++) {
        
        for(unsigned int n=0; n<h.sectors[i].size(); n++)
        {   
            h.lambdaX[i][n] = h.Hkx[i](n,n); 
            h.lambdaY[i][n] = h.Hky[i](n,n); 
            h.partialLambdaXX[i][n] = 0.0; 
            h.partialLambdaXY[i][n] = 0.0; 
            h.partialLambdaYY[i][n] = 0.0; 
            for(unsigned int m=0; m<h.sectors[i].size(); m++)
            {
                if(n==m) 
                {            
                    //nothing.
                }
                else {
                    if (abs(h.eigenValues[i][n] - h.eigenValues[i][m]) < 0.0001) 
                    {   
                        //nothing.
                    }
                    else
                    {   
                        h.partialLambdaXX[i][n] += (h.Hkx[i](n,m) * h.Hkx[i](m,n)) / (h.eigenValues[i][n] - h.eigenValues[i][m]);
                        h.partialLambdaXY[i][n] += (h.Hkx[i](n,m) * h.Hky[i](m,n)) / (h.eigenValues[i][n] - h.eigenValues[i][m]);
                        h.partialLambdaYY[i][n] += (h.Hky[i](n,m) * h.Hky[i](m,n)) / (h.eigenValues[i][n] - h.eigenValues[i][m]);
                    }
                }
            }
        }
        
        for(unsigned int n=0; n<h.sectors[i].size(); n++) h.lambdaXX[i][n] = h.Hkxkx[i](n,n)+2.0*h.partialLambdaXX[i][n]; 
        for(unsigned int n=0; n<h.sectors[i].size(); n++) h.lambdaXY[i][n] = h.Hkxky[i](n,n)+2.0*h.partialLambdaXY[i][n]; 
        for(unsigned int n=0; n<h.sectors[i].size(); n++) h.lambdaYY[i][n] = h.Hkyky[i](n,n)+2.0*h.partialLambdaYY[i][n]; 
    }
}



void kernelEvaluation(ParameterSet &p, HamiltonianByProducts &h, double pxc, double pyc, double result[]) {
    
    //ParameterSet p = *((ParameterSet*)userdata);
    //double pxc = x[0];
    //double pyc = x[1];
    
    calculateConductivityInPlace(p,h,pxc,pyc);

    complex<double> z( 0.0, p.ETA );
    
    complex<double> Akw(0.0);
    complex<double> sigma_xx( 0.0, 0.0 );
    complex<double> sigma_yy( 0.0, 0.0 );
    complex<double> sigma_xy( 0.0, 0.0 );
    complex<double> sigma_xx2( 0.0, 0.0 );
    complex<double> sigma_yy2( 0.0, 0.0 );
    complex<double> sigma_xy2( 0.0, 0.0 );
    double fermiDirac(0.0);
    
    
    for(unsigned int i=0; i<h.sectors.size(); i++) {
        
        for(unsigned int n=0; n<h.sectors[i].size(); n++)
        {   
            Akw = M_1_PI*imag(1.0/(z-h.eigenValues[i][n]));
            sigma_xx += h.lambdaX[i][n]*Akw*h.lambdaX[i][n]*Akw;
            sigma_yy += h.lambdaY[i][n]*Akw*h.lambdaY[i][n]*Akw;
            sigma_xy += (1./3.)*( h.lambdaX[i][n]*Akw*h.lambdaX[i][n]*Akw*h.lambdaYY[i][n]*Akw
                                 +h.lambdaY[i][n]*Akw*h.lambdaY[i][n]*Akw*h.lambdaXX[i][n]*Akw
                             -2.0*h.lambdaX[i][n]*Akw*h.lambdaY[i][n]*Akw*h.lambdaXY[i][n]*Akw);
            
            fermiDirac = 1.0/(exp(h.eigenValues[i][n]/p.ETA)+1.0);
            sigma_xx2 += h.lambdaXX[i][n]*fermiDirac;
            sigma_yy2 += h.lambdaYY[i][n]*fermiDirac;
            sigma_xy2 += (h.lambdaXX[i][n]*h.lambdaYY[i][n] - h.lambdaXY[i][n]*h.lambdaXY[i][n]) * fermiDirac;
            
        }
    }
    
    result[0] = real(sigma_xx);
    result[1] = real(sigma_yy);
    result[2] = real(sigma_xy);

    result[3] = real(sigma_xx2);
    result[4] = real(sigma_yy2);
    result[5] = real(sigma_xy2);
    
}

int gridIntegrateHall(ParameterSet &p, HamiltonianByProducts &h,double* integral, double* error, int verboseIntegrate=1){
    
    if (p.nambuTerm) throw runtime_error("Cannot compute conductivity for system with finite Gorkov-Green function");

    int ncomp=6;
    double result[6];
    
    int steps = 1 << (p.grid_nMin - 1);
    for(int n = 0; n<ncomp; n++) integral[n] = .0;
    
    bool continueFlag=false;
    do {
        steps *= 2;	
        for(int n = 0; n<ncomp; n++) error[n] = integral[n]; 

        for(int n = 0; n<ncomp; n++) integral[n] = .0;
        for(int i = 0; i < steps; i++) 
           for(int j = 0; j < steps; j++) {
               double kx = ((double)i) / ((double)steps);
               double ky = ((double)j) / ((double)steps);
               kernelEvaluation(p, h, kx, ky, result);
               for(int n = 0; n<ncomp; n++) integral[n] += result[n];
           }

        for(int n = 0; n<ncomp; n++) integral[n] /= (steps*steps) ;

        continueFlag=false;
        for(int n = 0; n<ncomp; n++) {
            error[n] -= integral[n];
            if(abs(error[n]) > p.grid_EPSREL*abs(integral[n]) ) {
                if(abs(error[n]) > p.grid_EPSABS){
                    if(steps < (1 << p.grid_nMax)){
                        if(n<3) continueFlag=true;  //convergence criterion only check for the first 3 value to integrate here, the one obtained with A^3 and not the integral by part.
                    }
                }
            }
            
        }
        //printf("\n\nsteps=%d", steps);			
        //for(int n = 0; n<ncomp; n++) printf(", val%d=% 4.3e err%d=% 4.2e", n, integral[n], n, error[n]); 
    } while(continueFlag);
    
    for(int n = 0; n<ncomp; n++) {
        integral[n] *=2.0/p.dim; //2 for spin
        error[n]    *=2.0/p.dim; //2 for spin
    }
    return steps*steps;
}


/*
void conductivity(ParameterSet &p){
    
    initCalculation(p);

    if (p.verbose>=1) cout << endl << "calculating density: " << endl;
    double integral[6]; //container for integral result
    double error[6];
    
    cubaIntegrateDavid(p,integral,error,p.verbose);
    double density = integral[0];
    
    if (p.verbose>=1) cout << endl << "calculating conductivity:" << endl;
    //int steps = cubaIntegrateHall(p,integral, error);
    int steps = gridIntegrateHall(p,integral, error);
    
    
    FILE *file = fopen("conductivity.out","w");
    //fprintf(file, "\n       mu,         n  sigma_xx  sigma_yy  sigma_xy        R_H       n_H \n");
    
    fprintf(file, "\n       mu         n      sigma_xx      error_xx      sigma_yy      error_yy      sigma_xy      error_xy      sigma_xx2      error_xx2      sigma_yy2      error_yy2      sigma_xy2      error_xy2       n_eval\n");
    fprintf(file, "% 8.6f % 8.6f % 8.6e % 8.6e % 8.6e % 8.6e % 8.6e % 8.6e % 8.6e % 8.6e % 8.6e % 8.6e % 8.6e % 8.6e %d\n",
                p.getAmplitudeOf("mu"), density, integral[0], error[0], integral[1], error[1], integral[2], error[2], integral[3], error[3], integral[4], error[4] , integral[5], error[5], steps);

//    fprintf(file, "% 8.6f, % 8.6f % 8.6f % 8.6f % 8.6f % 8.6f % 8.6f\n", p.getAmplitudeOf("mu"), density, integral[0], integral[1], integral[2], integral[2]/(integral[0]*integral[1]), -(integral[0]*integral[1])/integral[2] );
    fclose(file);
    if(p.verbose>=2){
        printf("\n       mu         n      sigma_xx      error_xx      sigma_yy      error_yy      sigma_xy      error_xy      sigma_xx2      error_xx2      sigma_yy2      error_yy2      sigma_xy2      error_xy2       n_eval\n");
        printf("% 8.6f % 8.6f % 8.6e % 8.6e % 8.6e % 8.6e % 8.6e % 8.6e % 8.6e % 8.6e % 8.6e % 8.6e % 8.6e % 8.6e %d\n",
                    p.getAmplitudeOf("mu"), density, integral[0], error[0], integral[1], error[1], integral[2], error[2], integral[3], error[3], integral[4], error[4] , integral[5], error[5], steps);
    }

}
*/


double printSector(ParameterSet &p) {
   
    HmMatrix<bool> hamiltonianTerms = checkHamiltonian(p);
    
    hamiltonianTerms.print();
    
    vector<set<int>> sectors= findSectors(hamiltonianTerms);
    
    for(unsigned int i=0; i<sectors.size(); i++)
    {   
        printf("sector%d\n",i);
        for(set<int>::iterator iter=sectors[i].begin(); iter!=sectors[i].end();++iter) cout<<(*iter)<<" "; cout << " "; fflush(stdout);
        cout << "\n";
    }
    cout << "\n";
    
    HmMatrix<bool> hamiltonianTerms2;
    hamiltonianTerms.print();
    for(unsigned int i=0; i<sectors.size(); i++)
    {   
        printf("sector%d\n",i);
        hamiltonianTerms.squareBloc(hamiltonianTerms2,sectors[i]);
        hamiltonianTerms2.print();
    }
    
    fflush(stdout);
    
    return 0.0;
};


void conductivity_loopMU(ParameterSet &p){
    
    //printSector(p);
    
    if(p.verbose >= 1) cout << endl << "Printing conductivity file: 'conductivity.out'" << endl;
    
    FILE *file = fopen("conductivity.out","w");
    fprintf(file, "\n       mu         n      sigma_xx      error_xx      sigma_yy      error_yy      sigma_xy      error_xy      sigma_xx2      error_xx2      sigma_yy2      error_yy2      sigma_xy2      error_xy2      n_eval\n");
    int N_loop = floor((p.loop_end-p.loop_start)/p.loop_incr)+1;
    
    double integral[6]; //container for integral result
    double error[6];
    
    HamiltonianByProducts h(p);
    for (int i=0; i < N_loop; i++){
        ParameterSet pLoop = p;
        double mu=p.loop_start+ i*p.loop_incr;
    
        if(pLoop.verbose >= 1) printf("\nmu = % 5.2f     %d %%",mu,(int)(100*(mu-pLoop.loop_start)/(double)(pLoop.loop_end-pLoop.loop_start)));
        
        pLoop.setAmplitudeOf("mu",mu);
        cubaIntegrateDavid(pLoop,integral, error, 0);
        double density = integral[0];
        
        
        //add this condition to prevent to calculate for nothing (in insulating phase).
        double tol = 0.00001, fractpart, intpart;
        fractpart = modf (density*pLoop.normalDim , &intpart);
        if(abs(fractpart)>tol){  
            //int steps = cubaIntegrateHall(p,integral, error, 0);
            int steps = gridIntegrateHall(pLoop, h,integral, error, 0);
            
            fprintf(file, "% 8.6f % 8.6f % 8.6e % 8.6e % 8.6e % 8.6e % 8.6e % 8.6e % 8.6e % 8.6e % 8.6e % 8.6e % 8.6e % 8.6e %d\n",
                    mu, density, integral[0], error[0], integral[1], error[1], integral[2], error[2], integral[3], error[3], integral[4], error[4] , integral[5], error[5], steps );
            fflush(file);
        }
        else{
            fprintf(file, "% 8.6f % 8.6f\n",
                    mu, density );
            fflush(file);
        }
    }
    
    fclose(file);
    if(p.verbose >= 1) cout << '\n' << "Conductivity file printed " << endl;
    
}





/*
void luttingerCalculate(ParameterSet &p){
    int Nx=p.mdc_numberOfKx;
    int Ny=p.mdc_numberOfKy;
    
    p.reset_mdc_data();
    initCalculation(p);
    
    for (int px=0; px<Nx/p.Lx; ++px) {
        for (int py=0; py<Ny/p.Ly; ++py) {
            double koff = 0.5;
            double pxc = (px+koff)*p.Lx/(double)Nx;
            double pyc = (py+koff)*p.Ly/(double)Ny;
            complex<double> z(p.OMEGA, p.ETA);
            
            calculateConductivityInPlace(p,pxc,pyc);

            //printf("%d %d %d\n", px, py, sectors.size());
            for(unsigned int i=0; i<sectors.size(); i++)
            for(unsigned int n=0; n<sectors[i].size(); n++)
            {
                //printf("%d %d %f", px, py, M_1_PI*real(1.0/(z-eigenValues[i][n])));
                p.mdc_data[Nx*px + py] += -M_1_PI*imag(1.0/(z-eigenValues[i][n]));
                if(eigenValues[i][n]<0.0) {
                    p.gor_data_re[Nx*px + py] += 1.0/p.dim;
                   if(real(lambdaXX[i][n]+lambdaYY[i][n])>0.0) p.gor_data_im[Nx*px + py] += 1.0/p.dim; 
                   else p.gor_data_im[Nx*px + py] -= 1.0/p.dim; 
                }
            }
        }
    }
}
*/

#endif
