//
//  dos.h
//  oneBuddy
//

#ifndef oneBuddy_dos_h
#define oneBuddy_dos_h

#include "build.h"
#include "cuba.h"

static HmMatrix<complex<double>> green;

static int dosIntegrand(const int* ndim, const double x[],
                     const int* ncomp, double result[], void* userdata) {
    double pxc = x[0];
    double pyc = x[1];
    ParameterSet p = *((ParameterSet*)userdata);
    complex<double> z(p.OMEGA, p.ETA);
    buildGreenInPlace(green, p, pxc, pyc, z);
    
    *result=0;
    
    for (int band=0; band < p.nbOfBands; ++band) {
        int fromIndex = band*p.nambuDim;
        *result += - M_1_PI*imag( green.partialTrace(p.normalDim, fromIndex) )/double(p.normalDim);
    }
    
    //// THE TRUE DOS NECESSITATE THIS PART ALSO USE NAMBU_DOS
//    z = complex<double>(-p.OMEGA, p.ETA);
//    buildGreenInPlace(green, p, pxc, pyc, z);
//    for (int band=0; band < p.nbOfBands; ++band) {
//        int fromIndex = band*p.nambuDim + p.normalDim;
//        *result +=   M_1_PI*imag( green.selfConj().partialTrace(p.normalDim, fromIndex) )/double(p.normalDim);
//    }
//    *result /= 2.0;
    
    return 0;
}

void dosCubaIntegrate(ParameterSet &p, double* integral){
    void* userdata=&p;
    int NDIM=2;                     // const    number of dimensions
    int ncomp=1;                    // const    number of components of the integrand (could be a vector) // here, it is the argument of var integral
    int nvec=1;                     // const    number of samples passed to integrand; usually 1 (here it can be larger than the one passed to the Integrand.. why? no one knows)
    double EPSREL=0.001;
    if (p.cuba_EPSREL!=0) EPSREL=p.cuba_EPSREL;
    double EPSABS=0.0001;
    if (p.cuba_EPSABS!=0) EPSABS=p.cuba_EPSABS;
    int verbose=p.cuba_verbose;                  // const    should be flags, but complicated, for verbose purpose, limits are from 0 to 3, higher values will throw other commands to Cuhre (refer to doc)
    int MINEVAL= (int)(2.5/(p.ETA*p.ETA));      //(1000/((p.ETA*p.ETA)/(0.05*0.05)));
    if (p.cuba_MINEVAL!=0) MINEVAL=p.cuba_MINEVAL;
    int MAXEVAL=400000;
    if (p.cuba_MAXEVAL!=0) MAXEVAL=p.cuba_MAXEVAL;
    int KEY=13;                     // const    7, 9, 11 or 13 (see doc) for 2D should be 13
    void* spin=NULL;                // const    deepshit to manage internal process (keep it NULL)
    int nregions;                   //      OUT actual number of subregion
    int neval;                      //      OUT number of evaluation
    int fail;                       //      OUT error flag (see doc)
    double error[ncomp];            //      OUT error
    double prob[ncomp];             //      OUT another unreliable error (see doc)
    
    Cuhre(NDIM, ncomp, dosIntegrand, userdata, nvec,
          EPSREL, EPSABS, verbose, MINEVAL, MAXEVAL, KEY, NULL, spin,
          &nregions, &neval, &fail, integral, error, prob);
}


void dosPoint(ParameterSet& p, ofstream& file) {
    double integral[1]; //container for integral result (1D value, could be multiD)
    dosCubaIntegrate(p,integral);
    file << p.OMEGA << "\t"<< *integral << "\t" << endl;
}

void dos(const ParameterSet& p){
    if(p.verbose >= 1) cout << endl << "printing DOS file" << endl;
    ofstream dos_file;
    dos_file.open("dos.out");
    
    for (auto& ww : p.loopValues) {
        if(p.verbose >= 1) {
            cout << '\r' << "omega = "<< setw(5) << ((abs(ww) > 0.001)? ww:0.0) << "\t";
            cout << setw(3) << (int)(100*(ww-p.loop_start)/(double)(p.loop_end-p.loop_start));
            cout << " % done"  << flush;
        }
        ParameterSet p_temp=p; p_temp.setSlavesLists();
        p_temp.OMEGA = ww;
        dosPoint(p_temp, dos_file);
    }
    
    dos_file.close();
    if(p.verbose >= 1) cout << '\r' << "DOS file printed " << setw(20) << " " << endl;
}
















static int nambuDOSIntegrand(const int* ndim, const double x[],
                        const int* ncomp, double result[], void* userdata) {
    double pxc = x[0];
    double pyc = x[1];
    ParameterSet p = *((ParameterSet*)userdata);
    complex<double> z(p.OMEGA, p.ETA);
    buildGreenInPlace(green, p, pxc, pyc, z);
    
    result[0]=0;
    result[1]=0;
    
    for (int band=0; band < p.nbOfBands; ++band) {
        int fromIndex = band*p.nambuDim;
        result[0] += - M_1_PI*imag( green.partialTrace(p.normalDim, fromIndex) )/double(p.normalDim);
        if (p.nambuTrigger){
            result[1] += - M_1_PI*imag( green.partialTrace(p.normalDim, fromIndex+p.normalDim) )/double(p.normalDim);
        }
    }
    return 0;
}

void nambuDOSCubaIntegrate(ParameterSet &p, double* integral){
    void* userdata=&p;
    int NDIM=2;                     // const    number of dimensions
    int ncomp=2;                    // const    number of components of the integrand (could be a vector) // here, it is the argument of var integral
    int nvec=1;                     // const    number of samples passed to integrand; usually 1 (here it can be larger than the one passed to the Integrand.. why? no one knows)
    double EPSREL=0.001;
    if (p.cuba_EPSREL!=0) EPSREL=p.cuba_EPSREL;
    double EPSABS=0.0001;
    if (p.cuba_EPSABS!=0) EPSABS=p.cuba_EPSABS;
    int verbose=p.cuba_verbose;                  // const    should be flags, but complicated, for verbose purpose, limits are from 0 to 3, higher values will throw other commands to Cuhre (refer to doc)
    int MINEVAL= (int)(2.5/(p.ETA*p.ETA));      //(1000/((p.ETA*p.ETA)/(0.05*0.05)));
    if (p.cuba_MINEVAL!=0) MINEVAL=p.cuba_MINEVAL;
    int MAXEVAL=400000;
    if (p.cuba_MAXEVAL!=0) MAXEVAL=p.cuba_MAXEVAL;
    int KEY=13;                     // const    7, 9, 11 or 13 (see doc) for 2D should be 13
    void* spin=NULL;                // const    deepshit to manage internal process (keep it NULL)
    int nregions;                   //      OUT actual number of subregion
    int neval;                      //      OUT number of evaluation
    int fail;                       //      OUT error flag (see doc)
    double error[ncomp];            //      OUT error
    double prob[ncomp];             //      OUT another unreliable error (see doc)
    
    Cuhre(NDIM, ncomp, nambuDOSIntegrand, userdata, nvec,
          EPSREL, EPSABS, verbose, MINEVAL, MAXEVAL, KEY, NULL, spin,
          &nregions, &neval, &fail, integral, error, prob);
}


void nambuDOSPoint(ParameterSet& p, ofstream& file) {
    double integral[2]; //container for integral result (1D value, could be multiD)
    nambuDOSCubaIntegrate(p,integral);
    file << p.OMEGA << "\t" << integral[0] << "\t" << integral[1] << endl;
}

void nambuDOS(const ParameterSet& p){
    if(p.verbose >= 1) cout << endl << "printing DOS file" << endl;
    ofstream dos_file;
    dos_file.open("dos.out");
    
    for (auto& ww : p.loopValues) {
        if(p.verbose >= 1) {
            cout << '\r' << "omega = "<< setw(5) << ((abs(ww) > 0.001)? ww:0.0) << "\t";
            cout << setw(3) << (int)(100*(ww-p.loop_start)/(double)(p.loop_end-p.loop_start));
            cout << " % done"  << flush;
        }
        ParameterSet p_temp=p; p_temp.setSlavesLists();
        p_temp.OMEGA = ww;
        nambuDOSPoint(p_temp, dos_file);
    }
    
    dos_file.close();
    if(p.verbose >= 1) cout << '\r' << "DOS file printed " << setw(20) << " " << endl;
}
































static vector<double> eD_eigenValues;
static HmMatrix<complex<double>> eD_eigenVectors;
static HmMatrix<complex<double>> eD_hamiltonian;

static int Integrand_eigenDOS(const int* ndim, const double x[],
                     const int* ncomp, double result[], void* userdata) {
    double pxc = x[0];
    double pyc = x[1];
    ParameterSet p = *((ParameterSet*)userdata);
    complex<double> z(p.OMEGA, p.ETA);
    
    eD_eigenValues.resize(p.dim);
    eD_eigenVectors.resize(p.dim);
    eD_hamiltonian.resize(p.dim);
    buildHamiltonianInPlace(eD_hamiltonian, p, pxc, pyc ,0,0);
    diagonalize(eD_hamiltonian, eD_eigenVectors, eD_eigenValues);
    
    for (int i=0; i<p.dim; i++){
        result[i] = -M_1_PI*imag( 1.0/(z - eD_eigenValues[i]) );
    }
    
    return 0;
}


void cubaIntegrate_eigenDOS(ParameterSet &p, double* integral){
    void* userdata=&p;
    int NDIM=2;                     // const    number of dimensions
    int ncomp=p.dim;                    // const    number of components of the integrand (could be a vector) // here, it is the argument of var integral
    int nvec=1;                     // const    number of samples passed to integrand; usually 1 (here it can be larger than the one passed to the Integrand.. why? no one knows)
    double EPSREL=0.001;
    if (p.cuba_EPSREL!=0) EPSREL=p.cuba_EPSREL;
    double EPSABS=0.0001;
    if (p.cuba_EPSABS!=0) EPSABS=p.cuba_EPSABS;
    int verbose=p.cuba_verbose;                  // const    should be flags, but complicated, for verbose purpose, limits are from 0 to 3, higher values will throw other commands to Cuhre (refer to doc)
    int MINEVAL= (int)(2.5/(p.ETA*p.ETA));      //(1000/((p.ETA*p.ETA)/(0.05*0.05)));
    if (p.cuba_MINEVAL!=0) MINEVAL=p.cuba_MINEVAL;
    int MAXEVAL=400000;
    if (p.cuba_MAXEVAL!=0) MAXEVAL=p.cuba_MAXEVAL;
    int KEY=13;                     // const    7, 9, 11 or 13 (see doc) for 2D should be 13
    void* spin=NULL;                // const    deepshit to manage internal process (keep it NULL)
    int nregions;                   //      OUT actual number of subregion
    int neval;                      //      OUT number of evaluation
    int fail;                       //      OUT error flag (see doc)
    double error[ncomp];            //      OUT error
    double prob[ncomp];             //      OUT another unreliable error (see doc)
    
    Cuhre(NDIM, ncomp, Integrand_eigenDOS, userdata, nvec,
          EPSREL, EPSABS, verbose, MINEVAL, MAXEVAL, KEY, NULL, spin,
          &nregions, &neval, &fail, integral, error, prob);
}



void eigenDOS(const ParameterSet& p){
    if(p.verbose >= 1) cout << endl << "printing eigenDOS file" << endl;
    ofstream dos_file;
    dos_file.open("eigenDOS.out");
    
    
    dos_file << "freq" << "\t";
    for (int i=0; i<p.dim; i++){
        dos_file << "eigenDOS " << i << "\t";
    }
    dos_file << endl;
    
    
    for (auto& ww : p.loopValues) {
        if(p.verbose >= 1) {
            cout << '\r' << "omega = "<< setw(5) << ((abs(ww) > 0.001)? ww:0.0) << "\t";
            cout << setw(3) << (int)(100*(ww-p.loop_start)/(double)(p.loop_end-p.loop_start));
            cout << " % done"  << flush;
        }
        ParameterSet p_temp=p; p_temp.setSlavesLists();
        p_temp.OMEGA = ww;
        
        
        double integral[p.dim];
        cubaIntegrate_eigenDOS(p_temp,integral);
        
        dos_file << ww << "\t";
        for (int i=0; i<p.dim; i++){
            dos_file << integral[i] << "\t";
        }
        dos_file << endl;
    }
    
    dos_file.close();
    if(p.verbose >= 1) cout << '\r' << "DOS file printed " << setw(20) << " " << endl;
}






#endif
