//
//  localDOS.h
//  oneBuddy
//

#ifndef oneBuddy_hoppingDOS_h
#define oneBuddy_hoppingDOS_h

#include "build.h"
#include "cuba.h"

struct hdosBasisIndex {
    int Lx,Ly;
    int i,j;
    int Rx,Ry,Rx2,Ry2;
    int ax,ay;
    int axtilde,aytilde;
    int band, toband;
    int cuba_i;
    int sgn;
    complex<double> phX(const PlaneVector& k) {return polar(1.0, -k.x * Lx * axtilde); }
    complex<double> phY(const PlaneVector& k) {return polar(1.0, -k.y * Ly * aytilde); }
};


vector<hdosBasisIndex> hdosBasis;


void buildHDOSBasis(const ParameterSet& p) {
    hdosBasis.clear();
    hdosBasisIndex bidx;
    bidx.Lx = p.Lx;
    bidx.Ly = p.Ly;
    int index = 0;
    for (int Rx=0; Rx<p.Lx; ++Rx)
    for (int Ry=0; Ry<p.Ly; ++Ry) {
        bidx.Rx = Rx;
        bidx.Ry = Ry;
        
        for (int band=0; band < p.nbOfBands; ++band)
        for (auto& neighbor : p.neighborList) {
            if (p.bandSelected !=0 and p.bandSelected != band+1) continue;
            int ax = neighbor.xNum; int ay = neighbor.yNum;
            bidx.ax = ax;
            bidx.ay = ay;
            
            
            int Rx2 = posMod(Rx+ax,p.Lx);
            int Ry2 = posMod(Ry+ay,p.Ly);
            
            
            bidx.Rx2 = Rx2;
            bidx.Ry2 = Ry2;
            
            int i,j;
            if (p.toBandSelected != p.bandSelected) {
                i = p.Ly* Rx + Ry                                   +((p.bandSelected-1)*p.nambuDim);
                j = p.Ly* Rx2 + Ry2   +((p.toBandSelected-1)*p.nambuDim);
                bidx.band = p.bandSelected;
                bidx.toband = p.toBandSelected;
            } else {
                i = p.Ly* Rx + Ry                                   +(band*p.nambuDim); // because element.band==0 or element.band-1 == band
                j = p.Ly* Rx2 + Ry2   +(band*p.nambuDim);
                bidx.band = band + 1;
                bidx.toband = band + 1;
            }
            int axtilde = floorDiv( Rx + ax , p.Lx );
            int aytilde = floorDiv( Ry + ay , p.Ly );
            bidx.axtilde = axtilde;
            bidx.aytilde = aytilde;
            bidx.i = i;
            bidx.j = j;
            bidx.cuba_i = index;
            bidx.sgn = 1;
            
            //cout << Rx << "," << Ry << "   " << i << "," << j << "   " << index << endl;
            hdosBasis.push_back(bidx);
            index += 1;
            if (i!=j) {
                bidx.axtilde = -axtilde;
                bidx.aytilde = -aytilde;
                bidx.i = j;
                bidx.j = i;
                bidx.cuba_i = index;
                
                bidx.Rx2 = Rx;
                bidx.Ry2 = Ry;
                bidx.Rx = Rx2;
                bidx.Ry = Ry2;
                bidx.sgn = -1;
                
                //cout << Rx << "," << Ry << "   " << i << "," << j << "   " << index << endl;
                hdosBasis.push_back(bidx);
                index += 1;
            }
        }
    }
}





static int hoppingDOSIntegrand(const int* ndim, const double x[],
                             const int* ncomp, double result[], void* userdata) {
    double pxc = x[0];
    double pyc = x[1];
    ParameterSet p = *((ParameterSet*)userdata);
    complex<double> z(p.OMEGA, p.ETA);
    HmMatrix<complex<double>> hdosGreen = buildGreenRepRR(p, pxc, pyc, z);
    
    double kx = 2*M_PI*( pxc/(double)p.Lx - 0.5 ); // are those -0.5 really necessary?
    double ky = 2*M_PI*( pyc/(double)p.Ly - 0.5 );
    PlaneVector k( kx , ky );
    
    for ( hdosBasisIndex& bi : hdosBasis) {
        result[bi.cuba_i] = -M_1_PI*imag( hdosGreen(bi.i,bi.j)*bi.phX(k)*bi.phY(k) );
    }
    
    return 0;
}

void hoppingDOScubaIntegrate(ParameterSet &p, double* integral) {
    void* userdata=&p;
    int NDIM=2;                     // const    number of dimensions
    int ncomp= p.hdos_size;                    // const    number of components of the integrand (could be a vector) // here, it is the argument of var integral
    int nvec=1;                     // const    number of samples passed to integrand; usually 1 (here it can be larger than the one passed to the Integrand.. why? no one knows)
    double EPSREL=0.001;
    if (p.cuba_EPSREL!=0) EPSREL=p.cuba_EPSREL;
    double EPSABS=0.0001;
    if (p.cuba_EPSABS!=0) EPSABS=p.cuba_EPSABS;
    int verbose=p.cuba_verbose;                  // const    should be flags, but complicated, for verbose purpose, limits are from 0 to 3, higher values will throw other commands to Cuhre (refer to doc)
    int MINEVAL= (int)(2.5/(p.ETA*p.ETA));      //(1000/((p.ETA*p.ETA)/(0.05*0.05)));
    if (p.cuba_MINEVAL!=0) MINEVAL=p.cuba_MINEVAL;
    int MAXEVAL=400000;
    if (p.cuba_MAXEVAL!=0) MAXEVAL=p.cuba_MAXEVAL;
    int KEY=13;                     // const    7, 9, 11 or 13 (see doc) for 2D should be 13
    void* spin=NULL;                // const    deepshit to manage internal process (keep it NULL)
    int nregions;                   //      OUT actual number of subregion
    int neval;                      //      OUT number of evaluation
    int fail;                       //      OUT error flag (see doc)
    double error[ncomp];            //      OUT error
    double prob[ncomp];             //      OUT another unreliable error (see doc)
    
    Cuhre(NDIM, ncomp, hoppingDOSIntegrand, userdata, nvec,
          EPSREL, EPSABS, verbose, MINEVAL, MAXEVAL, KEY, NULL, spin,
          &nregions, &neval, &fail, integral, error, prob);
}



void printHeader(ofstream& file) {
    for (int i=0; i < (int)hdosBasis.size()+3; i++)
        file << "#" << i+1 << "\t";
    file << endl << "w" << "\t";
    for ( hdosBasisIndex& bi : hdosBasis) {
        file << "("<<bi.Rx<<","<<bi.Ry<<")" << "-" << "("<<bi.Rx2<<","<<bi.Ry2<<")" << "\t";
    }
    file << "dos\thopp_avg";
    file << endl;
}

void computeData(ParameterSet& p) {
    
    p.delete_hdos_data();
    int XdataDim = 2*p.hdos_Xcells*p.Lx;
    int YdataDim = 2*p.hdos_Ycells*p.Ly;
    p.hdos_data.resize(XdataDim * YdataDim);
    p.hdos_data_neg.resize(XdataDim * YdataDim);
    
    double* integral = new double[p.hdos_size];
    hoppingDOScubaIntegrate(p, integral);
    
    p.hdos_data.resize(2*p.hdos_Xcells*p.Lx *2*p.hdos_Ycells*p.Ly);
    int dataDim = 2*p.hdos_Ycells*p.Ly;
    
    for ( hdosBasisIndex& bi : hdosBasis) {
        p.hdos_data[ dataDim* (2*bi.Rx+bi.ax) + 2*bi.Ry+bi.ay ] = integral[bi.cuba_i];
    }
    
    delete[] integral;
}

void printData(const ParameterSet& p, ofstream& file){
    int dataDim = 2*p.hdos_Ycells*p.Ly;
    double trace=0;
    double hopMean=0;
    file << p.OMEGA << "\t";
    for ( hdosBasisIndex& bi : hdosBasis) {
        if (bi.i == bi.j) {
            trace += p.hdos_data[ dataDim* (2*bi.Rx+bi.ax) + 2*bi.Ry+bi.ay ]/p.normalDim;
        } else {
            hopMean += p.hdos_data[ dataDim* (2*bi.Rx+bi.ax) + 2*bi.Ry+bi.ay ]/(p.hdos_size-p.normalDim);
        }
        file << p.hdos_data[ dataDim* (2*bi.Rx+bi.ax) + 2*bi.Ry+bi.ay ];
        file << "\t";
    }
    file << trace << "\t" << hopMean << "\t" << endl;
}

void hoppingDOS(ParameterSet& p) {
    if(p.verbose >= 1) cout << endl << "printing hoppingDOS.out file" << endl;
    ofstream file;
    file.open("hoppingDOS.out");
    
    buildHDOSBasis(p);
    printHeader(file);
    for (auto& ww : p.loopValues) {
        if(p.verbose >= 1) {
            cout << '\r' << "omega = "<< setw(5) << ((abs(ww) > 0.001)? ww:0.0) << "\t";
            cout << setw(3) << (int)(100*(ww-p.loop_start)/(double)(p.loop_end-p.loop_start));
            cout << " % done"  << flush;
        }
        ParameterSet p_temp=p; p_temp.setSlavesLists();
        p_temp.OMEGA = ww;
    
        computeData(p_temp);
        printData(p_temp,file);
    }
    
    
    file.close();
    if(p.verbose >= 1) cout << '\r' << "hoppingDOS.out file printed " << setw(20) << " " << endl;
    
    //// PRINT hMatrix.out
//    ofstream matrix_file;
//    matrix_file.open("hMatrix.out");
//    for (int ry=0; ry < dataDim; ++ry) {
//        for (int rx=0; rx < dataDim; ++rx) {
//            matrix_file << p.hdos_data[dataDim * rx + ry] << "\t";
//        }
//        matrix_file << endl;
//    }
//    matrix_file.close();
//    if(p.verbose >= 1) cout << '\r' << "hoppingDOS matrix file printed" << endl;
    
}




void hoppingDOScalculate(ParameterSet& p){
    buildHDOSBasis(p);
    
    //// COMPUTE
    double* integral = new double[p.hdos_size];
    hoppingDOScubaIntegrate(p, integral);
    
    //// CLEAR DATA
    int XdataDim = 2*p.hdos_Xcells*p.Lx;
    int YdataDim = 2*p.hdos_Ycells*p.Ly;
    p.delete_hdos_data();
    p.hdos_data.resize(XdataDim * YdataDim);
    p.hdos_data_neg.resize(XdataDim * YdataDim);
    
    
    //// FILL DATA
    for (int xCell=0 ; xCell < p.hdos_Xcells; xCell++)
    for (int yCell=0 ; yCell < p.hdos_Ycells; yCell++) {
        for ( hdosBasisIndex& bi : hdosBasis) {
            int xCoor = bi.Lx*2*xCell + 2*bi.Rx+bi.ax;
            int yCoor = bi.Ly*2*yCell + 2*bi.Ry+bi.ay;
            
            p.hdos_data[ YdataDim* xCoor + yCoor ] += integral[ bi.cuba_i ];
            
//            if (bi.i != bi.j) {
//                int cuba_i_of_i = bi.i * (int)(2*p.neighborList.size()-1);
//                p.hdos_data[ YdataDim* xCoor + yCoor ] += integral[ cuba_i_of_i ];
//                p.hdos_data[ YdataDim* xCoor + yCoor ] *= 0.5;
//            }
        }
    }
    
    //// ZMAP
    //p.zmap = true;
    if (p.zmap) {
        p.OMEGA *= -1;
        hoppingDOScubaIntegrate(p, integral);
        
        //// FILL DATA_neg
        for (int xCell=0 ; xCell < p.hdos_Xcells; xCell++)
        for (int yCell=0 ; yCell < p.hdos_Ycells; yCell++) {
            for ( hdosBasisIndex& bi : hdosBasis) {
                int xCoor = bi.Lx*2*xCell + 2*bi.Rx+bi.ax;
                int yCoor = bi.Ly*2*yCell + 2*bi.Ry+bi.ay;
                p.hdos_data_neg[ YdataDim* xCoor + yCoor ] += integral[ bi.cuba_i ];
                
//                if (bi.i != bi.j) {
//                    int cuba_i_of_i = bi.i * (int)(2*p.neighborList.size()-1);
//                    p.hdos_data_neg[ YdataDim* xCoor + yCoor ] += integral[ cuba_i_of_i ];
//                    p.hdos_data_neg[ YdataDim* xCoor + yCoor ] *= 0.5;
//                }
            }
        }
        p.OMEGA *= -1;
        
        //// divide
        for (int i=0 ; i < XdataDim * YdataDim ; i++) {
            if (abs(p.hdos_data_neg[i]) > 0.0001) p.hdos_data[i] /= p.hdos_data_neg[i];
            else p.hdos_data[i] = 0.0;
        }
    }
    
    //// SET MAX VAL
    
    p.hdos_max =0;
    if (p.hdos_max == 0) {
        double hopMax=0;
        double hopMin=100000;
        for (int i=0; i < XdataDim * YdataDim; i++){
            if (abs(1.2*p.hdos_data[i]) > hopMax) hopMax = abs(1.0*p.hdos_data[i]);
            if (abs(1.2*p.hdos_data[i]) < 0.00001) continue;
            if (abs(1.2*p.hdos_data[i]) < hopMin) hopMin = abs(0.5*p.hdos_data[i]);
        }
//        double hopMax=0;
//        for (int i=0; i < XdataDim * YdataDim; i++){
//            hopMax += abs(p.hdos_data[i])/(XdataDim*YdataDim*0.75);
//        }
        
        p.hdos_max = hopMax;
        p.hdos_min = hopMin;
        p.hdos_min = 0;
        
    }
    
    delete[] integral;
}



#endif
