//
//  interactive.h
//  OneBuddy
//

#ifndef OneBuddy_interactive_h
#define OneBuddy_interactive_h

#ifdef INTERACTIVE  // to avoid this part on Mammouth
// part used by stdin (keyboard capture)
#include <stdio.h>
#include <stdlib.h>
#include <sys/time.h>
#include <sys/types.h>
#include <unistd.h>
#include <fcntl.h>
#include <termios.h>
// part used by gnuplot
#include "gnuplot_c_mod.h"
#include "mdc.h"




void lineKind(int val) {
    static int lineKind=1; //initialized only on the first use of the function (static)
    if (lineKind!=val) {
        if (lineKind!=0) printf("\n");
        lineKind=val;
    }
    printf("\r");
    fflush(stdout);
}

void plotMDC(const ParameterSet &p, FILE *hImage=NULL){
    int Nx=p.mdc_numberOfKx - p.mdc_numberOfKx%p.Lx;
    int Ny=p.mdc_numberOfKy - p.mdc_numberOfKx%p.Ly;
    
    if(p.mdc==0) {gpc_plot_image(hImage,p.mdc_data,Nx,Ny,0.0,6.0,p.mdc,p.mdcFlip); }
    else if(p.mdc==1) {gpc_plot_image(hImage,p.gor_data_re,Nx,Ny,-4.0,4.0,p.mdc,p.mdcFlip); }
    else if(p.mdc==2) {gpc_plot_image(hImage,p.gor_data_im,Nx,Ny,-4.0,4.0,p.mdc,p.mdcFlip); }
}



#define KNRM  "\x1B[0m"
#define KRED  "\x1B[31m"
#define KGRN  "\x1B[32m"
#define KBOLD "\033[1m"

inline void chooseColor(float val1,float val2, bool bold=false){
    float tol =0.00001;
    if(val1-val2<-tol){printf(KRED);}
    else if(val1-val2>tol){printf(KGRN);}
    else {printf(KNRM);}
}

inline void printCompact(ParameterSet &p, ParameterSet &p0){       // could put this in p object
    
    /*
    if(p.lastUpdate==-3) printf(KBOLD);
    chooseColor(p.getAmplitudeOf("mu"),p0.getAmplitudeOf("mu"));
    printf("mu=% 5.2f  ",p.getAmplitudeOf("mu"));
    printf(KNRM);
    
    if(p.lastUpdate==-2) printf(KBOLD);
    chooseColor(p.OMEGA,p0.OMEGA);
    printf("w=% 5.2f  ",p.OMEGA);
    printf(KNRM);
    
    if(p.lastUpdate==-1) printf(KBOLD);
    chooseColor(p.ETA,p0.ETA);
    printf("eta=% 5.2f  ",p.ETA);
    printf(KNRM);
    */
    
    lineKind(1);
    
    char tab[1024];
    for (int i=0;i< (p.nbOfTerms>8? 8:p.nbOfTerms) ; i++){
        strcpy(tab, p.termList[i].identifier.c_str());
        if(p.lastUpdate==i) printf(KBOLD);
        chooseColor(p.termList[i].factor,p0.termList[i].factor);
        printf("%s=%5.3f  ",tab,real(p.termList[i].factor));   // MC: I prefer printf over cout because I can enforce a constant number of digits easily
        printf(KNRM);
    }
    chooseColor(0.0,0.0);
    
    fflush(stdout);
    
}



inline void updateAmplitude(ParameterSet &p, int index1, float step, ParameterSet &p0){   // could be somewhere
    p.lastUpdate=index1;
    if(index1 < p.nbOfTerms){
        p.termList[index1].incrAmplitude(step);
    }
    printCompact(p,p0);
}


inline void emptyBuffer()
{   
    int c;
    while ((c=getchar()) == ' ' && c == '.'){read( fileno( stdin ), &c, 1 );}
    fflush(stdin);
}

void interactive_mdc(ParameterSet p){
    // p was copied
    p.setSlavesLists();
    
    FILE *hImage;
    hImage = gpc_init_image ();
    
    p.verbose=0;
    p.mdc_gorgov = true;
    mdcCalculate(p);
    ParameterSet backup = p; backup.setSlavesLists();
    ParameterSet backupPrint = p; backupPrint.setSlavesLists();
    plotMDC(p,hImage);
    
    float step=0.01;
    
    printf("Interactive Mode\n");
    printf("resolution = %d by %d\n",p.mdc_numberOfKx,p.mdc_numberOfKy);
    printf("step = %1.2f\n",step);
    printf("Type 'h' for help. Controllable parameters are : \n");
    printCompact(p,backupPrint);
    printf("\n\n");
    printCompact(p,backupPrint);
    
    struct termios oldSettings, newSettings;
    
    tcgetattr( fileno( stdin ), &oldSettings );
    newSettings = oldSettings;
    newSettings.c_lflag &= (~ICANON & ~ECHO);
    tcsetattr( fileno( stdin ), TCSANOW, &newSettings );
    
    char increaseParamKeys[] =  { 'q', 'w', 'e', 'r', 'u', 'i', 'o', 'p', '\0' };
    char decreaseParamKeys[] =  { 'a', 's', 'd', 'f', 'j', 'k', 'l', ';', '\0' };
    
    while ( 1 )
    {
        fd_set set;
        struct timeval tv;
        
        tv.tv_sec = 10;
        tv.tv_usec = 0;
        
        FD_ZERO( &set );
        FD_SET( fileno( stdin ), &set );
        
        int res = select( fileno( stdin )+1, &set, NULL, NULL, &tv );
        if( res > 0 )
        {
            p.verbose=0;
            char c;
            char ch[2561];
            if (read( fileno( stdin ), &ch, 2561 )==1){
                c=ch[0];
                for(int key=0;key<8;key++)
                {
                         if(c==increaseParamKeys[key]) {updateAmplitude(p, key, +step,backupPrint);}
                    else if(c==decreaseParamKeys[key]) {updateAmplitude(p, key, -step,backupPrint);}
                }
                
                if(c=='1')      {p.ETA +=step; lineKind(2); printf("eta=% 3.3f%50s\r",p.ETA,""); fflush(stdout);}
                else if(c=='2' and p.ETA > 2*step) {p.ETA -=step; lineKind(2); printf("eta=% 3.3f%50s\r",p.ETA,""); fflush(stdout);}
                
                else if(c=='3') {p.OMEGA +=step; lineKind(3); printf("omega=% 3.3f%50s\r",p.OMEGA,""); fflush(stdout);}
                else if(c=='4') {p.OMEGA -=step; lineKind(3); printf("omega=% 3.3f%50s\r",p.OMEGA,""); fflush(stdout);}
                
                else if(c=='+') { p.mdc_numberOfKx+=100; p.mdc_numberOfKy+=100; lineKind(4); printf("resolution= %4d by %4d\r",p.mdc_numberOfKx,p.mdc_numberOfKy); fflush(stdout);}
                else if(c=='-') { p.mdc_numberOfKx-=100; p.mdc_numberOfKy-=100; lineKind(4); printf("resolution= %4d by %4d\r",p.mdc_numberOfKx,p.mdc_numberOfKy); fflush(stdout);}
                
                else if(c==')' and step < 1000) { step*=10; lineKind(5); printf("small steps=%1.3f%50s\r",step,""); fflush(stdout);}
                else if(c=='(' and step > 0.001) { step/=10; lineKind(5); printf("small steps=%1.3f%50s\r",step,""); fflush(stdout);}
                
                else if(c=='t') { p.mdcFlip= !(p.mdcFlip); lineKind(6); if(!p.mdcFlip) printf("Not Flipped.%50s\r",""); else printf("Flipped.%50s\r",""); plotMDC(p,hImage); fflush(stdout);}  //flip
                else if(c=='g') { p.mdc=(p.mdc+1)%3; lineKind(6); printf("MDC mode:%d%50s\r",p.mdc,""); plotMDC(p,hImage); fflush(stdout);} //gorkov
                
                else if(c=='y') { p.verbose=2; p.delete_mdc_data(); density(p); p.verbose=0; printCompact(p,backupPrint); }
                
                else if(c==' ') { mdcCalculate(p); plotMDC(p,hImage); lineKind(0); backupPrint=p; backupPrint.setSlavesLists(); printCompact(p,backupPrint); }
                //else if(c=='.') { luttingerCalculate(p); plotMDC(p,hImage); lineKind(0); backupPrint=p; backupPrint.setSlavesLists(); printCompact(p,backupPrint); }
                
                
                else if(c=='=') {
                    p = backup; p.setSlavesLists(); backupPrint=backup; backupPrint.setSlavesLists(); printf("\nReset parameter space to input file values\n");
                    mdcCalculate(p);
                    plotMDC(p,hImage);
                    step = 0.01;
                    printf("resolution = %d by %d\n",p.mdc_numberOfKx,p.mdc_numberOfKy);
                    printf("step = %1.2f\n",step);
                    printCompact(p,backupPrint);
                    printf("\n");
                }
                
                else if(c=='h') {
                    printf("\n\n----------------- oneBuddy Interactive Mode Help ------------------\n");
                    printf("Keyboard commands:\n");
                    printf("\tSPACE   - compute and plot mdc for selected paramters \n");
                    printf("\ty       - calculate density \n"); 
                    printf("\tt       - flip MDC \n"); 
                    printf("\tg       - cycle through mdc, gorkov real, gorkov imaginary \n"); 
                    printf("\t=       - reset interface and parameter space to input file values\n"); 
                    printf("\t+ -     - change resolution \n"); 
                    printf("\t) (     - change steps \n");
                    printf("\t1 2     - change eta \n");
                    printf("\t3 4     - change omega \n");
                    printf("\t.       - print Luttinger surface \n");
                    printf("\th       - print help \n\n");
                    printf("Resolution = %d by %d\n",p.mdc_numberOfKx,p.mdc_numberOfKy);
                    printf("Step = %1.2f\n",step);
                    printf("Controlling keys and corresponding parameters\n");
                    printf("aq          sw         de          fr           ju           ki         lo         ;p      \n");
                    printCompact(p,backupPrint);
                }
            }
        }
        else if( res < 0 )
        {
            perror( "select error\n" );
            break;
        }
    }
    tcsetattr( fileno( stdin ), TCSANOW, &oldSettings );
    gpc_close (hImage);
}








void plotHDOS(const ParameterSet &p, FILE *hImage=NULL){
    int Nx=2*p.hdos_Xcells*p.Lx;
    int Ny=2*p.hdos_Ycells*p.Ly;
    
    gpc_plot_image(hImage, p.hdos_data, Nx, Ny, p.hdos_min, p.hdos_max, p.zmap? 5:4, false);
}




void interactive_hoppingDOS( ParameterSet p ){
    p.setSlavesLists();
    
    FILE *hImage;
    hImage = gpc_init_image ();
    
    p.verbose=0;
    hoppingDOScalculate(p);
    ParameterSet backup = p; backup.setSlavesLists();
    ParameterSet backupPrint = p; backupPrint.setSlavesLists();
    plotHDOS(p,hImage);
    
    float step=0.01;
    
    printf("Interactive Mode for hoppingDOS\n");
    printf("cells = %d by %d\n",p.hdos_Xcells,p.hdos_Ycells);
    printf("step = %1.2f\n",step);
    printf("Type 'h' for help. Controllable parameters are : \n");
    printCompact(p,backupPrint);
    printf("\n\n");
    printCompact(p,backupPrint);
    
    struct termios oldSettings, newSettings;
    
    tcgetattr( fileno( stdin ), &oldSettings );
    newSettings = oldSettings;
    newSettings.c_lflag &= (~ICANON & ~ECHO);
    tcsetattr( fileno( stdin ), TCSANOW, &newSettings );
    
    char increaseParamKeys[] =  { 'q', 'w', 'e', 'r', 'u', 'i', 'o', 'p', '\0' };
    char decreaseParamKeys[] =  { 'a', 's', 'd', 'f', 'j', 'k', 'l', ';', '\0' };
    
    while ( 1 )
    {
        fd_set set;
        struct timeval tv;
        
        tv.tv_sec = 10;
        tv.tv_usec = 0;
        
        FD_ZERO( &set );
        FD_SET( fileno( stdin ), &set );
        
        int res = select( fileno( stdin )+1, &set, NULL, NULL, &tv );
        if( res > 0 )
        {
            p.verbose=0;
            char c;
            char ch[2561];
            if (read( fileno( stdin ), &ch, 2561 )==1){
                c=ch[0];
                for(int key=0;key<8;key++)
                {
                    if(c==increaseParamKeys[key]) {updateAmplitude(p, key, +step,backupPrint);}
                    else if(c==decreaseParamKeys[key]) {updateAmplitude(p, key, -step,backupPrint);}
                }
                
                if(c=='1')      {p.ETA +=step; lineKind(2); printf("eta=% 3.3f%50s\r",p.ETA,""); fflush(stdout);}
                else if(c=='2' and p.ETA > 2*step) {p.ETA -=step; lineKind(2); printf("eta=% 3.3f%50s\r",p.ETA,""); fflush(stdout);}
                
                else if(c=='3') {p.OMEGA +=step; lineKind(3); printf("omega=% 3.3f%50s\r",p.OMEGA,""); fflush(stdout);}
                else if(c=='4') {p.OMEGA -=step; lineKind(3); printf("omega=% 3.3f%50s\r",p.OMEGA,""); fflush(stdout);}
                else if(c=='5') {p.OMEGA *=-1.0; lineKind(3); printf("omega=% 3.3f%50s\r",p.OMEGA,""); fflush(stdout);}
                
                
                else if(c=='+') { p.hdos_Xcells+=1; p.hdos_Ycells+=1; lineKind(4); printf("cells= %4d by %4d\r",p.hdos_Xcells,p.hdos_Ycells); fflush(stdout);}
                else if(c=='-') { p.hdos_Xcells-=1; p.hdos_Ycells-=1; lineKind(4); printf("cells= %4d by %4d\r",p.hdos_Xcells,p.hdos_Ycells); fflush(stdout);}
                
                else if(c==')' and step < 1000) { step*=10; lineKind(5); printf("small steps=%1.3f%50s\r",step,""); fflush(stdout);}
                else if(c=='(' and step > 0.001) { step/=10; lineKind(5); printf("small steps=%1.3f%50s\r",step,""); fflush(stdout);}
                
                else if(c=='y') { p.verbose=2; p.delete_hdos_data(); density(p); p.verbose=0; printCompact(p,backupPrint); }
                
                else if(c=='g') { p.hdos_max=0; p.zmap=!p.zmap; lineKind(6); printf("hdos mode:%s-map%50s\r",p.zmap? "z":"g",""); fflush(stdout);} //gorkov
                else if(c=='t') { p.hdos_max=0; lineKind(6); printf("reset scale%50s",""); fflush(stdout);} //gorkov
                
                
                else if(c==' ') { hoppingDOScalculate(p); plotHDOS(p,hImage); backupPrint=p; backupPrint.setSlavesLists(); printf("\nprinted"); fflush(stdout);}
                
                
                else if(c=='=') {
                    p = backup; p.setSlavesLists(); backupPrint=backup; backupPrint.setSlavesLists(); printf("\nReset parameter space to input file values\n");
                    hoppingDOScalculate(p);
                    plotHDOS(p,hImage);
                    step = 0.01;
                    printf("cells = %d by %d\n",p.hdos_Xcells,p.hdos_Ycells);
                    printf("step = %1.2f\n",step);
                    printCompact(p,backupPrint);
                    printf("\n");
                }
                
                else if(c=='h') {
                    printf("\n\n----------------- oneBuddy Interactive hoppingDOS Help ------------------\n");
                    printf("Keyboard commands:\n");
                    printf("\tSPACE   - compute and plot hoppingDOS for selected paramters \n");
                    printf("\ty       - calculate density \n");
                    printf("\tg       - toggle between g-map and z-map \n");
                    printf("\t=       - reset interface and parameter space to input file values\n");
                    printf("\t+ -     - change number of cells \n");
                    printf("\t) (     - change steps \n");
                    printf("\t1 2     - change eta \n");
                    printf("\t3 4     - change omega \n");
                    printf("\th       - print help \n\n");
                    printf("Resolution = %d by %d\n",p.hdos_Xcells,p.hdos_Ycells);
                    printf("Step = %1.2f\n",step);
                    printf("Controlling keys and corresponding parameters\n");
                    printf("aq          sw         de          fr           ju           ki         lo         ;p      \n");
                    printCompact(p,backupPrint);
                }
            }
        }
        else if( res < 0 )
        {
            perror( "select error\n" );
            break;
        }
    }
    tcsetattr( fileno( stdin ), TCSANOW, &oldSettings );
    gpc_close (hImage);

}








#endif
#endif
