//
//  localDOS.h
//  oneBuddy
//

#ifndef oneBuddy_localDOS_h
#define oneBuddy_localDOS_h

#include "build.h"
#include "cuba.h"

static HmMatrix<complex<double>> ldos_green;

static int localDOSIntegrand(const int* ndim, const double x[],
                             const int* ncomp, double result[], void* userdata) {
    double pxc = x[0];
    double pyc = x[1];
    ParameterSet p = *((ParameterSet*)userdata);
    complex<double> z(p.OMEGA, p.ETA);
    buildGreenRepRRInPlace(ldos_green, p, pxc, pyc, z);
    
    *result=0;
    for (int i=0; i<p.normalDim; i++){
        result[i] = -M_1_PI*imag( ldos_green(i,i) );
    }
    
    return 0;
}

void localDOScubaIntegrate(ParameterSet &p, double* integral) {
    void* userdata=&p;
    int NDIM=2;                     // const    number of dimensions
    int ncomp= p.normalDim;                    // const    number of components of the integrand (could be a vector) // here, it is the argument of var integral
    int nvec=1;                     // const    number of samples passed to integrand; usually 1 (here it can be larger than the one passed to the Integrand.. why? no one knows)
    double EPSREL=0.001;
    if (p.cuba_EPSREL!=0) EPSREL=p.cuba_EPSREL;
    double EPSABS=0.0001;
    if (p.cuba_EPSABS!=0) EPSABS=p.cuba_EPSABS;
    int verbose=p.cuba_verbose;                  // const    should be flags, but complicated, for verbose purpose, limits are from 0 to 3, higher values will throw other commands to Cuhre (refer to doc)
    int MINEVAL= (int)(2.5/(p.ETA*p.ETA));      //(1000/((p.ETA*p.ETA)/(0.05*0.05)));
    if (p.cuba_MINEVAL!=0) MINEVAL=p.cuba_MINEVAL;
    int MAXEVAL=400000;
    if (p.cuba_MAXEVAL!=0) MAXEVAL=p.cuba_MAXEVAL;
    int KEY=13;                     // const    7, 9, 11 or 13 (see doc) for 2D should be 13
    void* spin=NULL;                // const    deepshit to manage internal process (keep it NULL)
    int nregions;                   //      OUT actual number of subregion
    int neval;                      //      OUT number of evaluation
    int fail;                       //      OUT error flag (see doc)
    double error[ncomp];            //      OUT error
    double prob[ncomp];             //      OUT another unreliable error (see doc)
    
    Cuhre(NDIM, ncomp, localDOSIntegrand, userdata, nvec,
          EPSREL, EPSABS, verbose, MINEVAL, MAXEVAL, KEY, NULL, spin,
          &nregions, &neval, &fail, integral, error, prob);
}





void printHeader(const int& dim, ofstream& file) {
    for (int i=0; i<dim+2; i++) {
        file << "#" << i+1 << "\t";
    }
    
    file << endl << "w" << "\t";
    for (int i=0; i<dim; i++) {
        file << "A" << i << i << "\t";
    }
    file << "dos" << "\t" << endl;
}




void printLocalDOS(ParameterSet& p, ofstream& file) {
    
    double* integral = new double[p.normalDim];
    localDOScubaIntegrate(p, integral);
    double trace=0;
    
    file << p.OMEGA << "\t";
    for (int i=0; i < p.normalDim; i++){
        trace += integral[i]/p.normalDim;
        file << integral[i] << "\t";
    }
    file << trace << "\t" << endl;
    
    delete[] integral;
}




void localDOS(ParameterSet& p) {
    if(p.verbose >= 1) cout << endl << "printing localDOS.out file" << endl;
    ofstream file;
    file.open("localDOS.out");
    
    printHeader(p.normalDim, file);
    printLocalDOS(p, file);
    
    file.close();
    if(p.verbose >= 1) cout << '\r' << "localDOS.out file printed " << setw(20) << " " << endl;
}


void localDOS_loopMU(const ParameterSet& p){
    if(p.verbose >= 1) cout << endl << "printing localDOS.out file" << endl;
    ofstream file;
    file.open("localDOS.out");
    
    printHeader(p.normalDim, file);
    
    for (auto& ww : p.loopValues) {
        if(p.verbose >= 1) {
            cout << '\r' << "omega = "<< setw(5) << ((abs(ww) > 0.001)? ww:0.0) << "\t";
            cout << setw(3) << (int)(100*(ww-p.loop_start)/(double)(p.loop_end-p.loop_start));
            cout << " % done"  << flush;
        }
        ParameterSet p_temp=p; p_temp.setSlavesLists();
        p_temp.OMEGA = ww;
        
        printLocalDOS(p_temp, file);
    }
    
    file.close();
    if(p.verbose >= 1) cout << '\r' << "localDOS.out file printed " << setw(20) << " " << endl;
}

#endif
