//
//  mdc.h
//  oneBuddy
//

#ifndef oneBuddy_mdc_h
#define oneBuddy_mdc_h

#include <iomanip>
#include "build.h"

void mdcCalculate(ParameterSet &p){
    if(p.verbose >= 1) cout << endl << "printing MDC file" << endl;
    int Nx=p.mdc_numberOfKx - p.mdc_numberOfKx%p.Lx;
    int Ny=p.mdc_numberOfKy - p.mdc_numberOfKx%p.Ly;
    
    p.reset_mdc_data();
    
    for (int px=0; px<Nx/p.Lx; ++px) {
        if(p.verbose >= 1) cout << '\r' << setw(3) << int((px*p.Lx)/(double)(Nx)*100) << "%% done" << flush;
        for (int py=0; py<Ny/p.Ly; ++py) {
            double koff = 0.5;
            double pxc = (px+koff)*p.Lx/(double)Nx;
            double pyc = (py+koff)*p.Ly/(double)Ny;
            complex<double> z(p.OMEGA, p.ETA);
            
            //HmMatrix<complex<double>> green(buildGreen(p, pxc, pyc, z));
            HmMatrix<complex<double>> green(buildGreen(p, pxc, pyc, z));
            
            for (int nx=0; nx<p.Lx; ++nx) {
                for (int ny=0; ny<p.Ly; ++ny) {
                    int kx          = (nx*Nx/p.Lx)+px;
                    int ky          = (ny*Ny/p.Ly)+py;
                    int diagIndex   = (p.Ly*nx+ny);
                    
                    double value    = -M_1_PI*imag(green(diagIndex,diagIndex));
                    double BZoffset = (p.mdc_tiles)? 0.2*((nx+ny)%2):0;
                    p.mdc_data[Ny*kx + ky] = value + BZoffset;
                    
                    if (p.nambuTrigger and p.mdc_gorgov) {
                        int gor_diag_i          = p.Ly*nx+ny + p.normalDim;
                        int gor_diag_j          = p.Ly*nx+ny;
                        double gor_value_re     = real(green(gor_diag_i,gor_diag_j));
                        double gor_value_im     = imag(green(gor_diag_i,gor_diag_j));
                        p.gor_data_re[Ny*kx+ky] = gor_value_re;
                        p.gor_data_im[Ny*kx+ky] = gor_value_im;
                    }
                }
            }
        }
    }
}


void mdcPrint(const ParameterSet &p){
    int Nx=p.mdc_numberOfKx - p.mdc_numberOfKx%p.Lx;
    int Ny=p.mdc_numberOfKy - p.mdc_numberOfKx%p.Ly;
    
    ofstream mdc_file, gor_file_re, gor_file_im;
    mdc_file.open("mdc[1,1].out");
    if (p.nambuTrigger and p.mdc_gorgov) {
        gor_file_re.open("gorkov_00_re.out");
        gor_file_im.open("gorkov_00_im.out");
    }
    int xStart = p.mdc_quarter? Nx/2:0;
    int yStart = p.mdc_quarter? Ny/2:0;
    for (int py=yStart; py<Ny; ++py) {
        for (int px=xStart; px<Nx; ++px) {
            mdc_file << p.mdc_data[Ny*px + py] << "\t";
            if (p.nambuTrigger and p.mdc_gorgov) {
                gor_file_re << p.gor_data_re[Nx*px + py] << "\t";
                gor_file_im << p.gor_data_im[Nx*px + py] << "\t";
            }
        }
        mdc_file << endl;
        if (p.nambuTrigger and p.mdc_gorgov) {
            gor_file_re << endl;
            gor_file_im << endl;
        }
    }
    mdc_file.close();
    if (p.nambuTrigger and p.mdc_gorgov) {
        gor_file_re.close();
        gor_file_im.close();
    }
    
    if (p.mdc_flipped) {
        ofstream r_mdc_file;
        r_mdc_file.open("flipped_mdc[1,1].out");
        for (int px=Nx-1; px>-1; --px) {
            for (int py=0; py<Ny; ++py) {
                r_mdc_file << p.mdc_data[Nx*px + py] << "\t";
            }
            r_mdc_file << endl;
        }
        r_mdc_file.close();
    }
    
    if(p.verbose >= 1) cout << '\r' << "MDC file printed" << endl;
    
}

void mdc(ParameterSet &p){
    mdcCalculate(p);
    mdcPrint(p);
}
















void RBZmdcCalculate(ParameterSet &p){
    if(p.verbose >= 1) cout << endl << "printing RBZ MDC file" << endl;
    int Nx=p.mdc_numberOfKx - p.mdc_numberOfKx%p.Lx;
    int Ny=p.mdc_numberOfKy - p.mdc_numberOfKx%p.Ly;
    
    p.reset_mdc_data();
    
    for (int px=0; px<Nx/p.Lx; ++px) {
        if(p.verbose >= 1) cout << '\r' << setw(3) << int((px*p.Lx)/(double)(Nx)*100) << "%% done" << flush;
        for (int py=0; py<Ny/p.Ly; ++py) {
            double koff = 0.5;
            double pxc = (px+koff)*p.Lx/(double)Nx;
            double pyc = (py+koff)*p.Ly/(double)Ny;
            complex<double> z(p.OMEGA, p.ETA);
            
            HmMatrix<complex<double>> green(buildGreen(p, pxc, pyc, z));
            
            for (int nx=0; nx<p.Lx; ++nx) {
                for (int ny=0; ny<p.Ly; ++ny) {
                    int diagIndex   = (p.Ly*nx+ny);
                    double value    = -M_1_PI*imag(green(diagIndex,diagIndex));
                    p.mdc_data[Ny*px + py] += value;
                    
                }
            }
        }
    }
}


void RBZmdcPrint(const ParameterSet &p){
    int Nx=p.mdc_numberOfKx - p.mdc_numberOfKx%p.Lx;
    int Ny=p.mdc_numberOfKy - p.mdc_numberOfKx%p.Ly;
    
    ofstream mdc_file;
    mdc_file.open("RBZmdc.out");
    for (int py=0; py<Ny/p.Ly; ++py) {
    for (int px=0; px<Nx/p.Lx; ++px) {
        mdc_file << p.mdc_data[Nx*px + py] << "\t";
        }
        mdc_file << endl;
    }
    mdc_file.close();
    
    if(p.verbose >= 1) cout << '\r' << "RBZ MDC file printed" << endl;
    
}


void RBZmdc(ParameterSet &p){
    RBZmdcCalculate(p);
    RBZmdcPrint(p);
}











#endif
