//
//  main.cpp
//  oneBuddy
//

#include <iostream>
#include "ParameterSet.h"
#include "mdc.h"
#include "dos.h"
#include "density.h"
#include "conductivity.h"
#include "spectrum.h"
#include "average.h"
#include "localDOS.h"
#include "butterfly.h"

#include "hoppingDOS.h"

//#define INTERACTIVE

#ifdef INTERACTIVE
#include "interactive.h"
#endif

int main(int argc, const char * argv[]) {
    ParameterSet p;
    p.read("OBpara.dat");
    p.selfUpdate();
    
    if (getenv("CUBACORES")==NULL) putenv((char*)"CUBACORES=0");
    
    if(argc>=3){
        string prm(argv[2]);
        if (prm=="eta") {p.ETA= (argc >= 4)? stod(argv[3]):p.ETA;}
        if (prm=="omega") {p.OMEGA= (argc >= 4)? stod(argv[3]):p.OMEGA;}
        p.setAmplitudeOf(prm, (argc >= 4)? stod(argv[3]):p.getAmplitudeOf(prm));
    }
    if (p.verbose >=2) p.print();
    
    cout << endl << "oneBuddy starting\n" << endl;
    
    if(argc>=2){
		string opt(argv[1]);
        if      (opt=="mdc")            mdc(p);
        else if (opt=="dos")            dos(p);
        else if (opt=="density")        density(p);
        //else if (opt=="conductivity")   conductivity(p);
        //else if (opt=="old")            conductivityOld(p);
        else if (opt=="spectrum")       spectrum(p);
        else if (opt=="average")        average(p);
        else if (opt=="localDOS")       localDOS(p);
        else if (opt=="RBZmdc")         RBZmdc(p);
        else if (opt=="butterfly")      butterfly(p);
        else if (opt=="nambuDOS")       nambuDOS(p);
        else if (opt=="hoppingDOS")     hoppingDOS(p);
#ifdef INTERACTIVE
        else if (opt=="hdi")            interactive_hoppingDOS(p);
		else if (opt=="i")              interactive_mdc(p);
        else if (opt=="interactive")    interactive_mdc(p);
#endif
        else {cout << "could not understand the task to do." << endl;}
    }
    else if (p.mdc_trigger)          mdc(p);
    else if (p.dos_trigger)          dos(p);
    else if (p.density_trigger)      density(p);
    else if (p.average_trigger)      average(p);
    else if(p.density_trigger_loop) density_loopMU(p);
    //else if (p.conductivity)         conductivity(p);
    else if (p.conductivityLoop)     conductivity_loopMU(p);
    //else if (p.conductivityLoop_parallel)     conductivity_loopMU_parallel(p);
    else if (p.spectrum_trigger)     spectrum(p);
    else if (p.localDOS_trigger)     localDOS(p);
    else if (p.localDOSLoop_trigger) localDOS_loopMU(p);
    else if (p.butterfly_trigger)    butterfly(p);
    else if (p.RBZmdc_trigger)       RBZmdc(p);
    else if (p.eigenDOS_trigger)     eigenDOS(p);
    else if (p.hdos_trigger)         hoppingDOS(p);
#ifdef INTERACTIVE
    else if (p.mdc_interactive_trigger) {interactive_mdc(p);}
#endif
    else {cout << "could not understand the task to do." << endl;}
    
    cout << endl << "oneBuddy over." << endl;
    return 0;
}
