//
//  spectrum.h
//  oneBuddy
//

#ifndef oneBuddy_spectrum_h
#define oneBuddy_spectrum_h

#include "build.h"


// Could be improved by using only the reduced Brillouin Zone
void spectrum(const ParameterSet& p) {
    if(p.verbose >= 1) cout << endl << "printing spectrum file" << endl;
    ofstream spectrum_file;
    spectrum_file.open("spectrum.out");
    
    for (int w=0; w < p.spectrum_wpoints; ++w) {
        double omega = p.spectrum_start_freq + w*p.spectrum_freq_incr;
        queue<int> kpoints_queue = p.spectrum_kpoints_queue;
        queue<double> kx_queue = p.spectrum_kx_queue;
        queue<double> ky_queue = p.spectrum_ky_queue;
        double kx1 = kx_queue.front(); kx_queue.pop();
        double ky1 = ky_queue.front(); ky_queue.pop();
        
        while(!kpoints_queue.empty()){
            double kpoints = kpoints_queue.front(); kpoints_queue.pop();
            double kx2 = kx_queue.front(); kx_queue.pop();
            double ky2 = ky_queue.front(); ky_queue.pop();
            double step_kx=(kx2-kx1)/kpoints;
            double step_ky=(ky2-ky1)/kpoints;
            
            for (int k=0; k < kpoints; ++k) {
                double pxc = (kx1+k*step_kx+0.5)*(double)p.Lx;
                double pyc = (ky1+k*step_ky+0.5)*(double)p.Ly;
                complex<double> z(omega, p.ETA);
                double value = -1/M_PI*imag( buildGreen(p, pxc, pyc, z)(0,0) );
                spectrum_file << value << "\t";
            }
            kx1 = kx2;
            ky1 = ky2;
        }
        spectrum_file << endl;
        if(p.verbose >= 1) cout << '\r' << setw(3) << "omega = " << omega << "   " << flush;
    }
    spectrum_file.close();
}

#endif
