//
//  testKnownSystems.cpp
//  oneBuddy
//
//  Created by Simon Verret on 2016-09-05.
//  Copyright (c) 2016 Simon Verret. All rights reserved.
//

#ifndef oneBuddy_testKnownSystems_cpp
#define oneBuddy_testKnownSystems_cpp


#include <unistd.h>
#include <stdio.h>
#include <string>
#include <cstdlib>
#include <sstream>
using namespace std;

#include "../test/catch.hpp"
#include "../src/ParameterSet.h"
#include "../src/build.h"
#include "../src/mdc.h"
#include "../src/spectrum.h"
#include "../src/dos.h"
#include "../src/density.h"
#include "../src/conductivity.h"
#include "../src/localDOS.h"

bool areFilesEqual(string path1, string path2){
    // may be slow, see " cplusplus.com/forum/general/94032/ " to speed up
    double tolerance = 0.0001;
    bool sameSoFar = true;
    ifstream f1; f1.open(path1);
    ifstream f2; f2.open(path2);
    string linef1;
    string linef2;
    while(getline(f1,linef1) and getline(f2,linef2)) {
        istringstream buffer1(linef1);
        istringstream buffer2(linef2);
        string word1;
        string word2;
        while(buffer1 >> word1 and buffer2 >> word2 and sameSoFar) {
            if (word1 != word2) {
                if (is_number(word1) and is_number(word2)) {
                    sameSoFar = (abs(stof(word1)-stof(word2)) < tolerance);
                } else {
                    sameSoFar = false;
                }
                
                if (!sameSoFar){
                    cout << "compared string " << word1 << " -vs- " << word2;
                    cout << ((is_number(word1) and is_number(word2))?" they were numbers ":" were not both numbers --> yielded FALSE") << endl;
                    if (is_number(word1) and is_number(word2)) {
                        cout << "compared floats " << stof(word1) << " -vs- " << stof(word2);
                        cout << " yielded difference " << abs(stof(word1)-stof(word2));
                        cout << " vs tolerance " << tolerance << "--> yielded FALSE" << endl;
                    }
                }
                
            }
        }
    }
    return sameSoFar;
}



bool fileExist(string filename) {
    return access(filename.c_str(),F_OK) != -1;
}

void compareOutFile(string ouputFile, string expectedOutputFile, string failingName) {
    if (not fileExist(expectedOutputFile)) {
        rename(ouputFile.c_str(), failingName.c_str());
        bool refFileDoNotExist=false;
        CHECK(refFileDoNotExist);
    } else {
    CHECK(areFilesEqual(ouputFile, expectedOutputFile));
    if (areFilesEqual(ouputFile, expectedOutputFile))
        remove(ouputFile.c_str());
    else
        rename(ouputFile.c_str(), failingName.c_str());
    }
}

ParameterSet setupSystem(const string& OBparaFile) {
    ParameterSet p;
    p.verbose=0;
    p.read(OBparaFile);
    p.selfUpdate();
    if (getenv("CUBACORES")==NULL) putenv((char*)"CUBACORES=0");
    return p;
}

string fLoc = "test/IOfiles/";



TEST_CASE( "mdc" ) {
    
    SECTION( "system A" ) {
        ParameterSet p = setupSystem(fLoc+"sysA_OBpara.dat");
        mdc(p);
        compareOutFile("mdc[1,1].out", fLoc+"sysA_expected_mdc.out", "mdcSYSA");
    }
    
    SECTION( "system B" ) {
        ParameterSet p = setupSystem(fLoc+"sysB_OBpara.dat");
        mdc(p);
        compareOutFile("mdc[1,1].out", fLoc+"sysB_expected_mdc.out", "mdcSYSB");
    }
    
    SECTION( "system C" ) {
        ParameterSet p = setupSystem(fLoc+"sysC_OBpara.dat");
        mdc(p);
        compareOutFile("mdc[1,1].out", fLoc+"sysC_expected_mdc.out", "mdcSYSC");
    }
    
    SECTION( "system D" ) {
        ParameterSet p = setupSystem(fLoc+"sysD_OBpara.dat");
        mdc(p);
        compareOutFile("mdc[1,1].out", fLoc+"sysD_expected_mdc.out", "mdcSYSD");
    }
    
}



TEST_CASE( "spectrum" ) {
    
    SECTION( "system A" ) {
        ParameterSet p = setupSystem(fLoc+"sysA_OBpara.dat");
        spectrum(p);
        compareOutFile("spectrum.out", fLoc+"sysA_expected_spectrum.out", "spectrumSYSA");
    }
    
    SECTION( "system C" ) {
        ParameterSet p = setupSystem(fLoc+"sysC_OBpara.dat");
        spectrum(p);
        compareOutFile("spectrum.out", fLoc+"sysC_expected_spectrum.out", "spectrumSYSC");
    }
    
    SECTION( "system E" ) {
        ParameterSet p = setupSystem(fLoc+"sysE_OBpara.dat");
        spectrum(p);
        compareOutFile("spectrum.out", fLoc+"sysE_expected_spectrum.out", "spectrumSYSE");
    }
    
}



TEST_CASE( "dos" ) {
    
    SECTION( "system A" ) {
        ParameterSet p = setupSystem(fLoc+"sysA_OBpara.dat");
        dos(p);
        compareOutFile("dos.out", fLoc+"sysA_expected_dos.out", "dosSYSA");
    }
    
    SECTION( "system B" ) {
        ParameterSet p = setupSystem(fLoc+"sysB_OBpara.dat");
        dos(p);
        compareOutFile("dos.out", fLoc+"sysB_expected_dos.out", "dosSYSB");
    }
    
    SECTION( "system D" ) {
        ParameterSet p = setupSystem(fLoc+"sysD_OBpara.dat");
        dos(p);
        compareOutFile("dos.out", fLoc+"sysD_expected_dos.out", "dosSYSD");
    }
    
}


TEST_CASE( "conductivity" ) {
    
    SECTION( "system A" ) {
        ParameterSet p = setupSystem(fLoc+"sysA_OBpara.dat");
        conductivity_loopMU(p);
        compareOutFile("conductivity.out", fLoc+"sysA_expected_conductivity.out", "conductivitySYSA");
    }
    
    SECTION( "system C" ) {
        ParameterSet p = setupSystem(fLoc+"sysC_OBpara.dat");
        conductivity_loopMU(p);
        compareOutFile("conductivity.out", fLoc+"sysC_expected_conductivity.out", "conductivitySYSC");
    }
    
}



TEST_CASE( "density" ) {
    
    SECTION( "system A" ) {
        ParameterSet p = setupSystem(fLoc+"sysA_OBpara.dat");
        density_loopMU(p);
        compareOutFile("n.out", fLoc+"sysA_expected_n.out", "densitySYSA");
    }
    
    SECTION( "system C" ) {
        ParameterSet p = setupSystem(fLoc+"sysC_OBpara.dat");
        density_loopMU(p);
        compareOutFile("n.out", fLoc+"sysC_expected_n.out", "densitySYSC");
    }
    
}



TEST_CASE( "localDOS" ) {
    
    SECTION( "system C" ) {
        ParameterSet p = setupSystem(fLoc+"sysC_OBpara.dat");
        localDOS_loopMU(p);
        compareOutFile("localDOS.out", fLoc+"sysC_expected_localDOS.out", "localDOSSYSC");
    }
    
    SECTION( "system F" ) {
        ParameterSet p = setupSystem(fLoc+"sysF_OBpara.dat");
        localDOS_loopMU(p);
        compareOutFile("localDOS.out", fLoc+"sysF_expected_localDOS.out", "localDOSSYSF");
    }
    
}


#endif
